@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: GEdit.c,v 1.1 1998/07/24 16:12:06 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: GEdit.c,v 1.11 1999/11/16 20:33:42 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: GEdit.c,v 1.1 1998/07/24 16:12:06 mleisher Exp $";
+static char rcsid[] = "$Id: GEdit.c,v 1.11 1999/11/16 20:33:42 mleisher Exp $";
 #endif
 #endif
 
@@ -103,9 +103,9 @@ XmuttGlyphEditWidget gw;
  */
 static void
 #ifndef _NO_PROTO
-_XmuttGlyphEditCursorColor(Widget w, int offset, XrmValue *val)
+_XmuttGlyphEditDefaultColor(Widget w, int offset, XrmValue *val)
 #else
-_XmuttGlyphEditCursorColor(w, offset, val)
+_XmuttGlyphEditDefaultColor(w, offset, val)
 Widget w;
 int offset;
 XrmValue *val;
@@ -134,11 +134,11 @@ Widget w;
     XmuttGlyphEditWidget gw;
     XRectangle hi[4];
 
-    if (!XtIsRealized(w))
-      return;
-
     gw = (XmuttGlyphEditWidget) w;
 
+    if (!XtIsRealized(w) || gw->primitive.highlight_thickness == 0)
+      return;
+
     /*
      * Top of window.
      */
@@ -323,10 +323,14 @@ Boolean force;
         if (gw->gedit.selgc != 0)
           XFreeGC(XtDisplay(w), gw->gedit.selgc);
         gcm = GCForeground|GCBackground|GCFunction;
-        gcv.foreground = gw->primitive.foreground;
+        gcv.foreground = gw->gedit.selpix;
         gcv.background = gw->core.background_pixel;
-        gcv.foreground ^= gcv.background;
-        gcv.function = GXxor;
+        if (gw->gedit.selpix != gw->primitive.foreground)
+          gcv.function = GXcopy;
+        else {
+            gcv.foreground ^= gcv.background;
+            gcv.function = GXxor;
+        }
         gw->gedit.selgc = XCreateGC(XtDisplay(w), XtWindow(w), gcm, &gcv);
     }
 }
@@ -355,6 +359,7 @@ Dimension *width, *height;
      */
     gw->gedit.pixel_size = gw->gedit.default_pixel_size;
 
+    *height = margin + ((gw->gedit.pixel_size + 4) * grid->grid_height);
     while (gw->gedit.pixel_size >= 2) {
         *height = margin + ((gw->gedit.pixel_size + 4) * grid->grid_height);
         if (*height < (dht >> 1))
@@ -383,9 +388,10 @@ Boolean select;
 {
     Widget w;
     int bpr, set;
-    short dx, dy;
+    short dx, dy, di, si;
     Dimension adj;
     bdf_glyph_grid_t *grid;
+    unsigned char *masks;
     XRectangle pix;
 
     w = (Widget) gw;
@@ -395,29 +401,48 @@ Boolean select;
 
     grid = gw->gedit.grid;
 
+    switch (grid->bpp) {
+      case 1: masks = onebpp; di = 7; break;
+      case 2: masks = twobpp; di = 3; break;
+      case 4: masks = fourbpp; di = 1; break;
+    }
+
     adj = (_XmuttGlyphEditPMargins(gw) >> 1) + 2;
-    bpr = (grid->grid_width + 7) >> 3;
+    bpr = ((grid->grid_width * grid->bpp) + 7) >> 3;
 
     pix.width = pix.height = gw->gedit.pixel_size + 1;
     pix.x = adj + ((gw->gedit.pixel_size + 4) * x) + 2;
     pix.y = adj + ((gw->gedit.pixel_size + 4) * y) + 2;
 
     if (select == True && grid->sel.width != 0) {
-        bpr = (grid->sel.width + 7) >> 3;
-        dx = x - grid->sel.x;
+        bpr = ((grid->sel.width * grid->bpp) + 7) >> 3;
         dy = y - grid->sel.y;
-        set = grid->sel.bitmap[(dy * bpr) + (dx >> 3)] & (0x80 >> (dx & 7));
+        dx = (x - grid->sel.x) * grid->bpp;
+        si = (dx & 7) / grid->bpp;
+        set = grid->sel.bitmap[(dy * bpr) + (dx >> 3)] & masks[si];
+        if (di > si)
+          set >>= (di - si) * grid->bpp;
     } else {
-        bpr = (grid->grid_width + 7) >> 3;
-        dx = x;
         dy = y;
-        set = grid->bitmap[(dy * bpr) + (dx >> 3)] & (0x80 >> (dx & 7));
+        dx = x * grid->bpp;
+        si = (dx & 7) / grid->bpp;
+        set = grid->bitmap[(dy * bpr) + (dx >> 3)] & masks[si];
+        if (di > si)
+          set >>= (di - si) * grid->bpp;
     }
 
-    if (set)
-      XFillRectangle(XtDisplay(w), XtWindow(w), gw->gedit.pixgc,
-                     pix.x, pix.y, pix.width, pix.height);
-    else
+    if (set) {
+        if (grid->bpp > 1) {
+            if (grid->bpp == 2)
+              XSetForeground(XtDisplay(w), gw->gedit.pixgc,
+                             gw->gedit.colors[set]);
+            else
+              XSetForeground(XtDisplay(w), gw->gedit.pixgc,
+                             gw->gedit.colors[set + 4]);
+        }
+        XFillRectangle(XtDisplay(w), XtWindow(w), gw->gedit.pixgc,
+                       pix.x, pix.y, pix.width, pix.height);
+    } else
       XClearArea(XtDisplay(w), XtWindow(w), pix.x, pix.y,
                  pix.width, pix.height, False);
 
@@ -593,6 +618,36 @@ Boolean clear;
       XDrawLine(XtDisplay(w), XtWindow(w), gw->gedit.gridgc,
                 frame.x + fxoff, frame.y + yoff,
                 frame.x + fxoff + frame.width, frame.y + yoff);
+
+    /*
+     * Draw the CAP_HEIGHT if indicated and exists.
+     */
+    if (gw->gedit.show_cap_height == True && grid->cap_height != 0) {
+        yoff = (gw->gedit.pixel_size + 4) * (grid->base_y - grid->cap_height);
+        if (clear == False)
+          XDrawLine(XtDisplay(w), XtWindow(w), gc,
+                    frame.x + fxoff, frame.y + yoff,
+                    frame.x + fxoff + frame.width, frame.y + yoff);
+        else
+          XDrawLine(XtDisplay(w), XtWindow(w), gw->gedit.gridgc,
+                    frame.x + fxoff, frame.y + yoff,
+                    frame.x + fxoff + frame.width, frame.y + yoff);
+    }
+
+    /*
+     * Draw the X_HEIGHT if indicated and exists.
+     */
+    if (gw->gedit.show_x_height == True && grid->x_height != 0) {
+        yoff = (gw->gedit.pixel_size + 4) * (grid->base_y - grid->x_height);
+        if (clear == False)
+          XDrawLine(XtDisplay(w), XtWindow(w), gc,
+                    frame.x + fxoff, frame.y + yoff,
+                    frame.x + fxoff + frame.width, frame.y + yoff);
+        else
+          XDrawLine(XtDisplay(w), XtWindow(w), gw->gedit.gridgc,
+                    frame.x + fxoff, frame.y + yoff,
+                    frame.x + fxoff + frame.width, frame.y + yoff);
+    }
 }
 
 static void
@@ -754,7 +809,7 @@ unsigned long *bytes;
      * Get the bytes used for the selection.
      */
     size = bcount = grid->sel.bytes >> 1;
-    size += sizeof(unsigned short) << 1;
+    size += sizeof(unsigned short) * 3;
     bp = bmap = (unsigned char *) XtMalloc(size);
 
     /*
@@ -762,11 +817,15 @@ unsigned long *bytes;
      * the width and height types are 16-bit values.
      */
     if (!bdf_little_endian()) {
+        *bp++ = (grid->bpp >> 8) & 0xff;
+        *bp++ = grid->bpp & 0xff;
         *bp++ = (wd >> 8) & 0xff;
         *bp++ = wd & 0xff;
         *bp++ = (ht >> 8) & 0xff;
         *bp++ = ht & 0xff;
     } else {
+        *bp++ = grid->bpp & 0xff;
+        *bp++ = (grid->bpp >> 8) & 0xff;
         *bp++ = wd & 0xff;
         *bp++ = (wd >> 8) & 0xff;
         *bp++ = ht & 0xff;
@@ -779,137 +838,6 @@ unsigned long *bytes;
     return bmap;
 }
 
-#if 0
-/*
- * Encode the character in BDF form (plain ASCII).
- */
-static char *
-#ifndef _NO_PROTO
-_XmuttGlyphEditEncodeBDF(XmuttGlyphEditWidget gw, unsigned long *bytes)
-#else
-_XmuttGlyphEditEncodeBDF(gw, bytes)
-XmuttGlyphEditWidget gw;
-unsigned long *bytes;
-#endif
-{
-    unsigned long len, rlen, bpr;
-    short x, y, wd, ht, nx;
-    unsigned short byte;
-    bdf_glyph_grid_t *grid;
-    double ps, rx, dw;
-    char *res, *rp, buf[128];
-
-    grid = gw->gedit.grid;
-    rlen = 0;
-
-    /*
-     * Add the STARTCHAR keyword with the glyph name.
-     */
-    sprintf(buf, "STARTCHAR %s\n", grid->name);
-    len = (unsigned long) strlen(buf);
-    res = (char *) XtMalloc(len);
-    (void) memcpy(res, buf, len);
-    rlen += len;
-
-    /*
-     * Add the glyph encoding.
-     */
-    if (grid->unencoded) {
-        res = (char *) XtRealloc(res, rlen + 12);
-        (void) memcpy(res + rlen, "ENCODING -1\n", 12);
-        rlen += 12;
-    } else {
-        sprintf(buf, "ENCODING %ld\n", grid->encoding);
-        len = (unsigned long) strlen(buf);
-        res = XtRealloc(res, rlen + len);
-        (void) memcpy(res + rlen, buf, len);
-        rlen += len;
-    }
-
-    /*
-     * Recalculate the scalable width before adding it because the glyph
-     * device width might have changed.
-     */
-    if (grid->modified) {
-        ps = (double) grid->point_size;
-        rx = (double) grid->resolution_x;
-        dw = (double) grid->dwidth;
-        grid->swidth = (unsigned short) ((dw * 72000.0) / (ps * rx));
-    }
-
-    /*
-     * Add the scalable width.
-     */
-    sprintf(buf, "SWIDTH %hd 0\n", grid->swidth);
-    len = (unsigned long) strlen(buf);
-    res = XtRealloc(res, rlen + len);
-    (void) memcpy(res + rlen, buf, len);
-    rlen += len;
-
-    /*
-     * Add the device width.
-     */
-    sprintf(buf, "DWIDTH %hd 0\n", grid->dwidth);
-    len = (unsigned long) strlen(buf);
-    res = XtRealloc(res, rlen + len);
-    (void) memcpy(res + rlen, buf, len);
-    rlen += len;
-
-    /*
-     * Add the glyph bounding box.
-     */
-    sprintf(buf, "BBX %hd %hd %hd %hd\n",
-            grid->glyph_bbx.width, grid->glyph_bbx.height,
-            grid->glyph_bbx.x_offset, grid->glyph_bbx.y_offset);
-    len = (unsigned long) strlen(buf);
-    res = XtRealloc(res, rlen + len);
-    (void) memcpy(res + rlen, buf, len);
-    rlen += len;
-
-    /*
-     * Add the BITMAP keyword.
-     */
-    res = XtRealloc(res, rlen + 7);
-    (void) memcpy(res + rlen, "BITMAP\n", 7);
-    rlen += 7;
-
-    /*
-     * Add the bitmap itself.
-     */
-    bpr = (grid->grid_width + 7) >> 3;
-
-    len = ((bpr << 1) * grid->glyph_bbx.height) + grid->glyph_bbx.height;
-    res = XtRealloc(res, rlen + len);
-    rp = res + rlen;
-
-    wd = grid->glyph_x + grid->glyph_bbx.width;
-    ht = grid->glyph_y + grid->glyph_bbx.height;
-    for (y = grid->glyph_y; y < ht; y++) {
-        for (byte = 0, x = grid->glyph_x; x < wd; x++) {
-            nx = x & 7;
-            byte |= (grid->bitmap[(y * bpr) + (x >> 3)] & (0x80 >> nx))
-                                                 << (7 - nx);
-            if (x > grid->glyph_x && nx == 0) {
-                sprintf(rp, "%02X", byte);
-                byte = 0;
-                rp += 2;
-            }
-        }
-        *rp++ = '\n';
-    }
-    rlen += len;
-
-    /*
-     * Add the ENDCHAR keyword.
-     */
-    res = XtRealloc(res, rlen + 9);
-    (void) memcpy(res + rlen, "ENDCHAR\n\0", 9);
-    *bytes = rlen + 9;
-
-    return res;
-}
-#endif
-
 static void
 #ifndef _NO_PROTO
 _XmuttGlyphEditOwnClipboard(Widget w)
@@ -1073,6 +1001,22 @@ Cardinal *num_args;
     (void) memset((char *) &gw->gedit.sel_start, 0, sizeof(XPoint));
     (void) memset((char *) &gw->gedit.sel_end, 0, sizeof(XPoint));
 
+    /*
+     * Initialize the current color index.
+     */
+    if (gw->gedit.grid) {
+        if ((gw->gedit.cidx = (1 << gw->gedit.grid->bpp) - 1) == 0)
+          gw->gedit.cidx = 1;
+    } else
+      gw->gedit.cidx = -1;
+
+    /*
+     * Initialize the last color to something different than the current color
+     * index so the first color will be set in the GC when the pixels are
+     * drawn.
+     */
+    gw->gedit.lcolor = -2;
+
     /*
      * Set the initial width and height.
      */
@@ -1215,8 +1159,26 @@ Cardinal *num_args;
             ngw->core.height = height;
         }
         bdf_free_glyph_grid(cgw->gedit.grid);
-    } else if (ngw->gedit.default_pixel_size !=
-               cgw->gedit.default_pixel_size) {
+
+        /*
+         * Adjust the current color index value if necessary.
+         */
+        if (ngw->gedit.cidx == -1) {
+            if ((ngw->gedit.cidx = (1 << ngw->gedit.grid->bpp) - 1) == 0)
+              ngw->gedit.cidx = 1;
+        } else if (ngw->gedit.grid->bpp == 1)
+          ngw->gedit.cidx = 1;
+        else if (ngw->gedit.cidx >= (1 << ngw->gedit.grid->bpp))
+          ngw->gedit.cidx = (1 << ngw->gedit.grid->bpp) - 1;
+
+        /*
+         * Set the last color to something different than the current color so
+         * the GC will be changed.
+         */
+        ngw->gedit.lcolor = -2;
+    }
+
+    if (ngw->gedit.default_pixel_size != cgw->gedit.default_pixel_size) {
         _XmuttGlyphEditPreferredGeometry(ngw, &width, &height);
         if (width != cgw->core.width || height != cgw->core.height) {
             ngw->core.width = width;
@@ -1225,6 +1187,10 @@ Cardinal *num_args;
         redo = True;
     }
 
+    if (ngw->gedit.show_x_height != cgw->gedit.show_x_height ||
+        ngw->gedit.show_cap_height != cgw->gedit.show_cap_height)
+      redo = True;
+
     return redo;
 }
 
@@ -1670,6 +1636,46 @@ short degrees;
     }
 }
 
+void
+#ifndef _NO_PROTO
+XmuttGlyphEditEmbolden(Widget w)
+#else
+XmuttGlyphEditEmbolden(w)
+Widget w;
+#endif
+{
+    XmuttGlyphEditWidget gw;
+    bdf_glyph_grid_t *grid;
+    bdf_bitmap_t image;
+    bdf_metrics_t metrics;
+    XmuttGlyphEditUpdateCallbackStruct cb;
+
+    _XmuttGlyphEditCheckClass(w);
+
+    gw = (XmuttGlyphEditWidget) w;
+    grid = gw->gedit.grid;
+
+    if (bdf_grid_embolden(grid)) {
+        XClearArea(XtDisplay(w), XtWindow(w), 0, 0, 0, 0, True);
+
+        bdf_grid_image(grid, &image);
+        cb.reason = XmuttGLYPHEDIT_IMAGE_CHANGE;
+        cb.event = 0;
+        cb.image = &image;
+        XtCallCallbackList(w, gw->gedit.change, (XtPointer) &cb);
+        if (image.bytes > 0)
+          free((char *) image.bitmap);
+
+        /*
+         * Alert the client the glyph was modified.
+         */
+        XmuttGlyphEditMetrics(w, &metrics, False);
+        cb.reason = XmuttGLYPHEDIT_GLYPH_MODIFIED;
+        cb.metrics = &metrics;
+        XtCallCallbackList(w, gw->gedit.modified, (XtPointer) &cb);
+    }
+}
+
 void
 #ifndef _NO_PROTO
 XmuttGlyphEditFlip(Widget w, short direction)
@@ -1807,6 +1813,23 @@ int op;
     }
 }
 
+void
+#ifndef _NO_PROTO
+XmuttGlyphEditChangeColor(Widget w, int color)
+#else
+XmuttGlyphEditChangeColor(w, color)
+Widget w;
+int color;
+#endif
+{
+    XmuttGlyphEditWidget gw;
+
+    _XmuttGlyphEditCheckClass(w);
+
+    gw = (XmuttGlyphEditWidget) w;
+    gw->gedit.cidx = color;
+}
+
 void
 #ifndef _NO_PROTO
 XmuttGlyphEditInsertBitmap(Widget w, bdf_bitmap_t *bitmap)
@@ -1967,6 +1990,7 @@ String filename;
                                          &xh, &yh)) != BitmapSuccess)
       return res;
 
+    bmap.bpp = 1;
     bmap.width = (unsigned short) wd;
     bmap.height = (unsigned short) ht;
     bpr = (bmap.width + 7) >> 3;
@@ -2003,6 +2027,7 @@ String filename;
     FILE *out;
     bdf_glyph_grid_t *grid;
     int cnt, x, y, sx, sy, bpr, i, byte;
+    unsigned char *masks;
     char *np, *ep, name[256];
 
     _XmuttGlyphEditCheckClass(w);
@@ -2019,6 +2044,12 @@ String filename;
     gw = (XmuttGlyphEditWidget) w;
     grid = gw->gedit.grid;
 
+    switch (grid->bpp) {
+      case 1: masks = onebpp; break;
+      case 2: masks = twobpp; break;
+      case 4: masks = fourbpp; break;
+    }
+
     /*
      * Determine the name to use for the bitmap fields based on the filename.
      */
@@ -2052,16 +2083,18 @@ String filename;
      * Now dump the bitmap.
      */
     sy = grid->glyph_y;
-    bpr = (grid->grid_width + 7) >> 3;
-    fprintf(out, "static unsigned char %s_bits[] = {", name);
+    bpr = ((grid->grid_width * grid->bpp) + 7) >> 3;
+    fprintf(out, "static char %s_bits[] = {", name);
     for (cnt = y = 0; y < grid->glyph_bbx.height; y++, sy++) {
-        for (sx = grid->glyph_x, x = 0; x < grid->glyph_bbx.width; ) {
+        for (sx = grid->glyph_x * grid->bpp, x = 0;
+             x < grid->glyph_bbx.width; ) {
             /*
              * Accumulate bits until there are 8.
              */
             for (byte = i = 0; x < grid->glyph_bbx.width && i < 8;
-                 i++, x++, sx++) {
-                if (grid->bitmap[(sy * bpr) + (sx >> 3)] & (0x80 >> (sx & 7)))
+                 i++, x++, sx += grid->bpp) {
+                if (grid->bitmap[(sy * bpr) + (sx >> 3)] &
+                    masks[(sx & 7) / grid->bpp])
                   byte |= 1 << i;
             }
             /*
@@ -2259,18 +2292,22 @@ Widget w;
         bp = data;
 
         if (!bdf_little_endian()) {
+            image.bpp = (*bp++ << 8) & 0xff00;
+            image.bpp |= *bp++;
             image.width = (*bp++ << 8) & 0xff00;
             image.width |= *bp++;
             image.height = (*bp++ << 8) & 0xff00;
             image.height |= *bp++;
         } else {
+            image.bpp = *bp++ & 0xff;
+            image.bpp |= (*bp++ << 8) & 0xff00;
             image.width = *bp++ & 0xff;
             image.width |= (*bp++ << 8) & 0xff00;
             image.height = *bp++ & 0xff;
             image.height |= (*bp++ << 8) & 0xff00;
         }
 
-        image.bytes = ((image.width + 7) >> 3) * image.height;
+        image.bytes = (((image.width * image.bpp) + 7) >> 3) * image.height;
         image.bitmap = bp;
 
         image.x = gw->gedit.last_x;
@@ -2540,16 +2577,29 @@ Cardinal *num_params;
         /*
          * Drawing.
          */
-        switch (event->xbutton.button) {
-          case Button1:
-            if ((changed = bdf_grid_set_pixel(grid, x, y)))
-              _XmuttGlyphEditDrawPixel(gw, x, y, False); break;
-          case Button2:
-            if ((changed = bdf_grid_invert_pixel(grid, x, y)))
-              _XmuttGlyphEditDrawPixel(gw, x, y, False); break;
-          case Button3:
-            if ((changed = bdf_grid_clear_pixel(grid, x, y)))
-              _XmuttGlyphEditDrawPixel(gw, x, y, False); break;
+        if (event->type == ButtonPress) {
+            switch (event->xbutton.button) {
+              case Button1:
+                if ((changed = bdf_grid_set_pixel(grid, x, y, gw->gedit.cidx)))
+                  _XmuttGlyphEditDrawPixel(gw, x, y, False);
+                break;
+              case Button2:
+                if ((changed = bdf_grid_invert_pixel(grid, x, y,
+                                                     gw->gedit.cidx)))
+                  _XmuttGlyphEditDrawPixel(gw, x, y, False);
+                break;
+              case Button3:
+                if ((changed = bdf_grid_clear_pixel(grid, x, y)))
+                  _XmuttGlyphEditDrawPixel(gw, x, y, False);
+                break;
+            }
+        } else if (event->type == KeyPress) {
+            /*
+             * Current assume that all key presses are the same as inverting
+             * a pixel.
+             */
+            if ((changed = bdf_grid_invert_pixel(grid, x, y, gw->gedit.cidx)))
+              _XmuttGlyphEditDrawPixel(gw, x, y, False);
         }
 
         /*
@@ -2686,8 +2736,12 @@ Cardinal *num_params;
     cb.event = event;
     cb.x = x - grid->base_x;
     cb.y = -(y - grid->base_y) - 1;
+    cb.color = bdf_grid_color_at(grid, x, y);
     XtCallCallbackList(w, gw->gedit.move, (XtPointer) &cb);
 
+    ix = gw->gedit.last_x;
+    iy = gw->gedit.last_y;
+
     /*
      * Set the current coordinates as the last known.
      */
@@ -2707,10 +2761,10 @@ Cardinal *num_params;
          * Drawing.
          */
         if (event->xmotion.state & Button1Mask) {
-            if ((changed = bdf_grid_set_pixel(grid, x, y)))
+            if ((changed = bdf_grid_set_pixel(grid, x, y, gw->gedit.cidx)))
               _XmuttGlyphEditDrawPixel(gw, x, y, False);
         } else if (event->xmotion.state & Button2Mask) {
-            if ((changed = bdf_grid_invert_pixel(grid, x, y)))
+            if ((changed = bdf_grid_invert_pixel(grid, x, y, gw->gedit.cidx)))
               _XmuttGlyphEditDrawPixel(gw, x, y, False);
         } else if (event->xmotion.state & Button3Mask) {
             if ((changed = bdf_grid_clear_pixel(grid, x, y)))
@@ -2776,7 +2830,7 @@ Cardinal *num_params;
         /*
          * A move or copy is in progress.
          */
-        if (bdf_has_selection(grid, &ix, &iy, 0, 0) &&
+        if (bdf_has_selection(grid, 0, 0, 0, 0) &&
             bdf_grid_shift(grid, x - ix, y - iy)) {
             _XmuttGlyphEditDrawGlyph(w);
             bdf_grid_image(grid, &image);
@@ -2926,6 +2980,43 @@ Cardinal *num_params;
     }
 }
 
+static void
+#ifndef _NO_PROTO
+ColorChange(Widget w, XEvent *event, String *params, Cardinal *num_params)
+#else
+ColorChange(w, event, params, num_params)
+Widget w;
+XEvent *event;
+String *params;
+Cardinal *num_params;
+#endif
+{
+    int n;
+    XmuttGlyphEditWidget gw;
+    XmuttGlyphEditUpdateCallbackStruct cb;
+
+    gw = (XmuttGlyphEditWidget) w;
+
+    if (gw->gedit.grid->bpp == 1)
+      return;
+
+    n = (1 << gw->gedit.grid->bpp) - 1;
+
+    switch (params[0][0]) {
+      case 'd': gw->gedit.cidx = (gw->gedit.cidx + 1) & n; break;
+      case 'l':
+        if (gw->gedit.cidx == 0)
+          gw->gedit.cidx = n;
+        else
+          gw->gedit.cidx--;
+        break;
+    }
+    cb.reason = XmuttGLYPHEDIT_COLOR_CHANGE;
+    cb.event = event;
+    cb.color = gw->gedit.cidx;
+    XtCallCallbackList(w, gw->gedit.opswitch, (XtPointer) &cb);
+}
+
 static void
 #ifndef _NO_PROTO
 ShiftGrid(Widget w, XEvent *event, String *params, Cardinal *num_params)
@@ -3121,46 +3212,42 @@ static char default_trans[] = "\
 <SelClr>: handle-clipboard(lose)\n\
 <FocusIn>: gain-focus()\n\
 <FocusOut>: lose-focus()\n\
+Shift Ctrl ~Meta ~Alt<Key>Return: start-action()\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Left: shift(left)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Up: shift(up)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Down: shift(down)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>KP_Right: shift(right)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>KP_Left: shift(left)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>KP_Up: shift(up)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>KP_Down: shift(down)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfRight: shift(right)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfLeft: shift(left)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfUp: shift(up)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfDown: shift(down)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>9: rotate(-)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>0: rotate(+)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>-: flip(horizontal)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>=: flip(vertical)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>comma: color-change(lighter)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>period: color-change(darker)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>z: color-change(lighter)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>x: color-change(darker)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunCopy: handle-clipboard(copy)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunCut: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunPaste: handle-clipboard(paste)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfCopy: handle-clipboard(copy)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfCut: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfPaste: handle-clipboard(paste)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfBackSpace: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfDelete: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>BackSpace: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Delete: handle-clipboard(cut)\n\
+Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(prev)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(next)\n\
 ~Shift ~Ctrl ~Meta ~Alt<BtnDown>: start-action()\n\
 ~Shift ~Ctrl ~Meta ~Alt<Motion>: extend-action()\n\
 ~Shift ~Ctrl ~Meta ~Alt<BtnUp>: end-action()\n\
 Shift ~Ctrl ~Meta ~Alt<Btn2Down>: handle-clipboard(paste)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Right: traverse(right)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Left: traverse(left)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Up: traverse(up)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Down: traverse(down)\n\
-~Shift Ctrl ~Meta ~Alt<Key>KP_Right: traverse(right)\n\
-~Shift Ctrl ~Meta ~Alt<Key>KP_Left: traverse(left)\n\
-~Shift Ctrl ~Meta ~Alt<Key>KP_Up: traverse(up)\n\
-~Shift Ctrl ~Meta ~Alt<Key>KP_Down: traverse(down)\n\
-Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(prev)\n\
-~Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(next)\n\
-<Key>Right: shift(right)\n\
-<Key>Left: shift(left)\n\
-<Key>Up: shift(up)\n\
-<Key>Down: shift(down)\n\
-<Key>KP_Right: shift(right)\n\
-<Key>KP_Left: shift(left)\n\
-<Key>KP_Up: shift(up)\n\
-<Key>KP_Down: shift(down)\n\
-<Key>osfRight: shift(right)\n\
-<Key>osfLeft: shift(left)\n\
-<Key>osfUp: shift(up)\n\
-<Key>osfDown: shift(down)\n\
-<Key>9: rotate(-)\n\
-<Key>0: rotate(+)\n\
-<Key>-: flip(horizontal)\n\
-<Key>=: flip(vertical)\n\
-<Key>SunCopy: handle-clipboard(copy)\n\
-<Key>SunCut: handle-clipboard(cut)\n\
-<Key>SunPaste: handle-clipboard(paste)\n\
-<Key>osfCopy: handle-clipboard(copy)\n\
-<Key>osfCut: handle-clipboard(cut)\n\
-<Key>osfPaste: handle-clipboard(paste)\n\
-<Key>osfBackSpace: handle-clipboard(cut)\n\
-<Key>osfDelete: handle-clipboard(cut)\n\
-<Key>BackSpace: handle-clipboard(cut)\n\
-<Key>Delete: handle-clipboard(cut)\n\
 ";
 
 #else /* R5 or lower */
@@ -3169,38 +3256,39 @@ static char default_trans[] = "\
 <SelClr>: handle-clipboard(lose)\n\
 <FocusIn>: gain-focus()\n\
 <FocusOut>: lose-focus()\n\
+Shift Ctrl ~Meta ~Alt<Key>Return: start-action()\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Right: shift(right)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Left: shift(left)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Up: shift(up)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Down: shift(down)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfRight: shift(right)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfLeft: shift(left)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfUp: shift(up)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfDown: shift(down)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>9: rotate(-)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>0: rotate(+)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>-: flip(horizontal)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>=: flip(vertical)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>comma: color-change(lighter)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>period: color-change(darker)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>z: color-change(lighter)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>x: color-change(darker)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunCopy: handle-clipboard(copy)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunCut: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>SunPaste: handle-clipboard(paste)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfCopy: handle-clipboard(copy)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfCut: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfPaste: handle-clipboard(paste)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfBackSpace: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>osfDelete: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>BackSpace: handle-clipboard(cut)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Delete: handle-clipboard(cut)\n\
+Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(prev)\n\
+~Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(next)\n\
 ~Shift ~Ctrl ~Meta ~Alt<BtnDown>: start-action()\n\
 ~Shift ~Ctrl ~Meta ~Alt<Motion>: extend-action()\n\
 ~Shift ~Ctrl ~Meta ~Alt<BtnUp>: end-action()\n\
 Shift ~Ctrl ~Meta ~Alt<Btn2Down>: handle-clipboard(paste)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Right: traverse(right)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Left: traverse(left)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Up: traverse(up)\n\
-~Shift Ctrl ~Meta ~Alt<Key>Down: traverse(down)\n\
-Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(prev)\n\
-~Shift ~Ctrl ~Meta ~Alt<Key>Tab: traverse(next)\n\
-<Key>Right: shift(right)\n\
-<Key>Left: shift(left)\n\
-<Key>Up: shift(up)\n\
-<Key>Down: shift(down)\n\
-<Key>osfRight: shift(right)\n\
-<Key>osfLeft: shift(left)\n\
-<Key>osfUp: shift(up)\n\
-<Key>osfDown: shift(down)\n\
-<Key>9: rotate(-)\n\
-<Key>0: rotate(+)\n\
-<Key>-: flip(horizontal)\n\
-<Key>=: flip(vertical)\n\
-<Key>SunCopy: handle-clipboard(copy)\n\
-<Key>SunCut: handle-clipboard(cut)\n\
-<Key>SunPaste: handle-clipboard(paste)\n\
-<Key>osfCopy: handle-clipboard(copy)\n\
-<Key>osfCut: handle-clipboard(cut)\n\
-<Key>osfPaste: handle-clipboard(paste)\n\
-<Key>osfBackSpace: handle-clipboard(cut)\n\
-<Key>osfDelete: handle-clipboard(cut)\n\
-<Key>BackSpace: handle-clipboard(cut)\n\
-<Key>Delete: handle-clipboard(cut)\n\
 ";
 
 #endif /* R5 or lower */
@@ -3216,6 +3304,7 @@ static XtActionsRec actions[] = {
     {"extend-action", ExtendAction},
     {"end-action", EndAction},
     {"handle-clipboard", HandleClipboard},
+    {"color-change", ColorChange},
 };
 
 static XtResource resources[] = {
@@ -3261,8 +3350,8 @@ static XtResource resources[] = {
         XmRPixel,
         sizeof(Pixel),
         XtOffset(XmuttGlyphEditWidget, gedit.selpix),
-        XmRString,
-        (XtPointer) "gold",
+        XmRCallProc,
+        (XtPointer) _XmuttGlyphEditDefaultColor,
     },
     {
         XmNcursorColor,
@@ -3271,7 +3360,7 @@ static XtResource resources[] = {
         sizeof(Pixel),
         XtOffset(XmuttGlyphEditWidget, gedit.cursorpix),
         XmRCallProc,
-        (XtPointer) _XmuttGlyphEditCursorColor,
+        (XtPointer) _XmuttGlyphEditDefaultColor,
     },
     {
         XmNimageChangeCallback,
@@ -3318,6 +3407,33 @@ static XtResource resources[] = {
         XmRImmediate,
         (XtPointer) 10,
     },
+    {
+        XmNshowXHeight,
+        XmCShowXHeight,
+        XmRBoolean,
+        sizeof(Boolean),
+        XtOffset(XmuttGlyphEditWidget, gedit.show_x_height),
+        XmRImmediate,
+        (XtPointer) False,
+    },
+    {
+        XmNshowCapHeight,
+        XmCShowCapHeight,
+        XmRBoolean,
+        sizeof(Boolean),
+        XtOffset(XmuttGlyphEditWidget, gedit.show_cap_height),
+        XmRImmediate,
+        (XtPointer) False,
+    },
+    {
+        XmNcolorList,
+        XmCColorList,
+        XmRPointer,
+        sizeof(unsigned long *),
+        XtOffset(XmuttGlyphEditWidget, gedit.colors),
+        XmRImmediate,
+        (XtPointer) 0,
+    },
 };
 
 externaldef(xmuttglypheditclassrec)
