@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,11 +24,11 @@
 #define _h_XmuttGEdit
 
 /*
- * $Id: GEdit.h,v 1.1 1998/07/24 16:12:07 mleisher Exp $
+ * $Id: GEdit.h,v 1.5 1999/09/14 17:40:45 mleisher Exp $
  */
 
 #include <Xm/Xm.h>
-#include "bdf.h"
+#include "bdfP.h"
 #include "GEShared.h"
 
 #ifdef __cplusplus
@@ -90,6 +90,15 @@ typedef struct _XmuttGlyphEditRec      *XmuttGlyphEditWidget;
 #define XmNdefaultPixelSize "defaultPixelSize"
 #define XmCDefaultPixelSize "DefaultPixelSize"
 
+#define XmNshowXHeight "showXHeight"
+#define XmCShowXHeight "ShowXHeight"
+
+#define XmNshowCapHeight "showCapHeight"
+#define XmCShowCapHeight "ShowCapHeight"
+
+#define XmNcolorList "colorList"
+#define XmCColorList "ColorList"
+
 /**************************************************************************
  *
  * Callback macros and structures.
@@ -103,6 +112,7 @@ typedef struct _XmuttGlyphEditRec      *XmuttGlyphEditWidget;
 #define XmuttGLYPHEDIT_POINTER_MOVED    -2
 #define XmuttGLYPHEDIT_OPERATION_CHANGE -3
 #define XmuttGLYPHEDIT_GLYPH_MODIFIED   -4
+#define XmuttGLYPHEDIT_COLOR_CHANGE     -5
 
 /*
  * Callback structures.
@@ -115,6 +125,7 @@ typedef struct {
     bdf_metrics_t *metrics;
     Position x;
     Position y;
+    int color;
 } XmuttGlyphEditUpdateCallbackStruct;
 
 /**************************************************************************
@@ -210,6 +221,11 @@ extern void XmuttGlyphEditRotate __((Widget w, short degrees));
  */
 extern void XmuttGlyphEditShear __((Widget w, short degrees));
 
+/*
+ * Make the glyph bold.
+ */
+extern void XmuttGlyphEditEmbolden __((Widget w));
+
 /*
  * Flip the bitmap horizontally or vertically.
  */
@@ -220,6 +236,11 @@ extern void XmuttGlyphEditFlip __((Widget w, short direction));
  */
 extern void XmuttGlyphEditChangeOperation __((Widget w, int op));
 
+/*
+ * Change the current color.
+ */
+extern void XmuttGlyphEditChangeColor __((Widget w, int color));
+
 /*
  * Insert a bitmap from some outside source.
  */
