@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: GEditTB.c,v 1.1 1998/07/24 16:12:08 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: GEditTB.c,v 1.4 1999/05/03 17:17:49 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: GEditTB.c,v 1.1 1998/07/24 16:12:08 mleisher Exp $";
+static char rcsid[] = "$Id: GEditTB.c,v 1.4 1999/05/03 17:17:49 mleisher Exp $";
 #endif
 #endif
 
@@ -32,31 +32,6 @@ static char rcsid[] = "$Id: GEditTB.c,v 1.1 1998/07/24 16:12:08 mleisher Exp $";
 #include "GEditTBP.h"
 #include "bitmaps.h"
 
-static unsigned char bitswap[] = {
-    0x00, 0x80, 0x40, 0xc0, 0x20, 0xa0, 0x60, 0xe0, 0x10, 0x90, 0x50, 0xd0,
-    0x30, 0xb0, 0x70, 0xf0, 0x08, 0x88, 0x48, 0xc8, 0x28, 0xa8, 0x68, 0xe8,
-    0x18, 0x98, 0x58, 0xd8, 0x38, 0xb8, 0x78, 0xf8, 0x04, 0x84, 0x44, 0xc4,
-    0x24, 0xa4, 0x64, 0xe4, 0x14, 0x94, 0x54, 0xd4, 0x34, 0xb4, 0x74, 0xf4,
-    0x0c, 0x8c, 0x4c, 0xcc, 0x2c, 0xac, 0x6c, 0xec, 0x1c, 0x9c, 0x5c, 0xdc,
-    0x3c, 0xbc, 0x7c, 0xfc, 0x02, 0x82, 0x42, 0xc2, 0x22, 0xa2, 0x62, 0xe2,
-    0x12, 0x92, 0x52, 0xd2, 0x32, 0xb2, 0x72, 0xf2, 0x0a, 0x8a, 0x4a, 0xca,
-    0x2a, 0xaa, 0x6a, 0xea, 0x1a, 0x9a, 0x5a, 0xda, 0x3a, 0xba, 0x7a, 0xfa,
-    0x06, 0x86, 0x46, 0xc6, 0x26, 0xa6, 0x66, 0xe6, 0x16, 0x96, 0x56, 0xd6,
-    0x36, 0xb6, 0x76, 0xf6, 0x0e, 0x8e, 0x4e, 0xce, 0x2e, 0xae, 0x6e, 0xee,
-    0x1e, 0x9e, 0x5e, 0xde, 0x3e, 0xbe, 0x7e, 0xfe, 0x01, 0x81, 0x41, 0xc1,
-    0x21, 0xa1, 0x61, 0xe1, 0x11, 0x91, 0x51, 0xd1, 0x31, 0xb1, 0x71, 0xf1,
-    0x09, 0x89, 0x49, 0xc9, 0x29, 0xa9, 0x69, 0xe9, 0x19, 0x99, 0x59, 0xd9,
-    0x39, 0xb9, 0x79, 0xf9, 0x05, 0x85, 0x45, 0xc5, 0x25, 0xa5, 0x65, 0xe5,
-    0x15, 0x95, 0x55, 0xd5, 0x35, 0xb5, 0x75, 0xf5, 0x0d, 0x8d, 0x4d, 0xcd,
-    0x2d, 0xad, 0x6d, 0xed, 0x1d, 0x9d, 0x5d, 0xdd, 0x3d, 0xbd, 0x7d, 0xfd,
-    0x03, 0x83, 0x43, 0xc3, 0x23, 0xa3, 0x63, 0xe3, 0x13, 0x93, 0x53, 0xd3,
-    0x33, 0xb3, 0x73, 0xf3, 0x0b, 0x8b, 0x4b, 0xcb, 0x2b, 0xab, 0x6b, 0xeb,
-    0x1b, 0x9b, 0x5b, 0xdb, 0x3b, 0xbb, 0x7b, 0xfb, 0x07, 0x87, 0x47, 0xc7,
-    0x27, 0xa7, 0x67, 0xe7, 0x17, 0x97, 0x57, 0xd7, 0x37, 0xb7, 0x77, 0xf7,
-    0x0f, 0x8f, 0x4f, 0xcf, 0x2f, 0xaf, 0x6f, 0xef, 0x1f, 0x9f, 0x5f, 0xdf,
-    0x3f, 0xbf, 0x7f, 0xff,
-};
-
 /*
  * Sizes for the button shapes.
  */
@@ -190,7 +165,7 @@ Dimension *width, *height;
 
     margin = _XmuttGEditTBoxPMargins(gw) + 4;
 
-    *width = margin + (3 * (_GETB_TOGGLE_SIZE + 4)) + 4;
+    *width = 50 + margin + (3 * (_GETB_TOGGLE_SIZE + 4)) + 4;
     *height = margin + (_GETB_TOGGLE_SIZE + 6) +
         ((5 * _GETB_BUTTON_SIZE) + 8);
 
@@ -533,6 +508,150 @@ Boolean force;
     }
 }
 
+static void
+#ifndef _NO_PROTO
+_XmuttGEditTBoxDrawColors(Widget w)
+#else
+_XmuttGEditTBoxDrawColors(w)
+Widget w;
+#endif
+{
+    XmuttGlyphEditToolboxWidget gw;
+    int i, s, e;
+    _XmuttGEditTBoxColor *cp;
+    XRectangle cr;
+
+    gw = (XmuttGlyphEditToolboxWidget) w;
+
+    if (gw->gedittb.colors == 0 || gw->gedittb.gimage->bpp == 1)
+      return;
+
+    if ((gw->gedittb.gimage->bpp == 2 && gw->gedittb.colors[0] == ~0) ||
+        (gw->gedittb.gimage->bpp == 4 && gw->gedittb.colors[4] == ~0))
+      return;
+
+    if (gw->gedittb.gimage->bpp == 2) {
+        s = 0;
+        e = 4;
+    } else {
+        s = 4;
+        e = 20;
+    }
+
+    cp = gw->gedittb.cols;
+
+    cr.width = gw->gedittb.cbw;
+    cr.height = gw->gedittb.cbh;
+
+    for (i = s; i < e; i++, cp++) {
+        cr.x = cp->x;
+        cr.y = cp->y;
+        /*
+         * If this is the current color index, put a box around it.
+         */
+        if (i - s == gw->gedittb.cidx)
+          XDrawRectangle(XtDisplay(w), XtWindow(w), gw->gedittb.gc,
+                         cr.x - 1, cr.y - 1, cr.width + 3, cr.height + 3);
+        XSetForeground(XtDisplay(w), gw->gedittb.fillgc,
+                       gw->gedittb.colors[i]);
+        XFillRectangle(XtDisplay(w), XtWindow(w), gw->gedittb.fillgc,
+                       cr.x + 1, cr.y + 1, cr.width, cr.height);
+    }
+    XSetForeground(XtDisplay(w), gw->gedittb.fillgc,
+                   gw->core.background_pixel);
+}
+
+static void
+#ifndef _NO_PROTO
+_XmuttGEditTBoxSelectColor(Widget w, int which)
+#else
+_XmuttGEditTBoxSelectColor(w, which)
+Widget w;
+int which;
+#endif
+{
+    XmuttGlyphEditToolboxWidget gw;
+    XRectangle rect;
+
+    gw = (XmuttGlyphEditToolboxWidget) w;
+
+    /*
+     * Locate the current color and clear the rectangle.
+     */
+    rect.x = gw->gedittb.cols[gw->gedittb.cidx].x - 1;
+    rect.y = gw->gedittb.cols[gw->gedittb.cidx].y - 1;
+    rect.width = gw->gedittb.cbw + 3;
+    rect.height = gw->gedittb.cbh + 3;
+    if (XtIsRealized(w)) {
+        XSetForeground(XtDisplay(w), gw->gedittb.gc,
+                       gw->core.background_pixel);
+        XDrawRectangle(XtDisplay(w), XtWindow(w), gw->gedittb.gc,
+                       rect.x, rect.y, rect.width, rect.height);
+    }
+
+    /*
+     * Locate the new color and draw the rectangle.
+     */
+    gw->gedittb.cidx = which;
+    rect.y = gw->gedittb.cols[gw->gedittb.cidx].y - 1;
+    if (XtIsRealized(w)) {
+        XSetForeground(XtDisplay(w), gw->gedittb.gc, gw->primitive.foreground);
+        XDrawRectangle(XtDisplay(w), XtWindow(w), gw->gedittb.gc,
+                       rect.x, rect.y, rect.width, rect.height);
+    }
+}
+
+static void
+#ifndef _NO_PROTO
+_XmuttGEditTBoxGetPixels(XmuttGlyphEditToolboxWidget gw, bdf_bitmap_t *im,
+                       int color)
+#else
+_XmuttGEditTBoxGetPixels(gw, im, color)
+XmuttGlyphEditToolboxWidget gw;
+bdf_bitmap_t *im;
+int color;
+#endif
+{
+    int byte;
+    unsigned short x, y, bpr, si, di, nx;
+    unsigned char *masks;
+
+    gw->gedittb.points_used = 0;
+
+    switch (im->bpp) {
+      case 1: masks = onebpp; di = 7; break;
+      case 2: masks = twobpp; di = 3; break;
+      case 4: masks = fourbpp; di = 1; break;
+    }
+
+    bpr = ((im->width * im->bpp) + 7) >> 3;
+    for (y = 0; y < im->height; y++) {
+        for (nx = x = 0; x < im->width; x++, nx += im->bpp) {
+            si = (nx & 7) / im->bpp;
+
+            byte = im->bitmap[(y * bpr) + (nx >> 3)] & masks[si];
+            if (di > si)
+              byte >>= (di - si) * im->bpp;
+            if (byte == color) {
+                if (gw->gedittb.points_used == gw->gedittb.points_size) {
+                    if (gw->gedittb.points_size == 0)
+                      gw->gedittb.points =
+                          (XPoint *) XtMalloc(sizeof(XPoint) * 64);
+                    else
+                      gw->gedittb.points = (XPoint *)
+                          XtRealloc((char *) gw->gedittb.points,
+                                    sizeof(XPoint) *
+                                    (gw->gedittb.points_size + 64));;
+                    gw->gedittb.points_size += 64;
+                }
+                gw->gedittb.points[gw->gedittb.points_used].x = x + im->x + 2;
+                gw->gedittb.points[gw->gedittb.points_used].y = y + im->y + 2;
+                gw->gedittb.points_used++;
+            }
+        }
+    }
+}
+
 static void
 #ifndef _NO_PROTO
 _XmuttGEditTBoxDrawImage(Widget w)
@@ -541,6 +660,8 @@ _XmuttGEditTBoxDrawImage(w)
 Widget w;
 #endif
 {
+    int color;
+    short si, ei;
     XmuttGlyphEditToolboxWidget gw;
     bdf_bitmap_t *im;
 
@@ -549,22 +670,43 @@ Widget w;
     if (!XtIsRealized(w) || (im = gw->gedittb.gimage) == 0)
       return;
 
-    /*
-     * Check to see if the pixmap exists for the image or not.  Create it if
-     * necessary.
-     */
-    if (gw->gedittb.image == 0)
-      gw->gedittb.image =
-          XCreateBitmapFromData(XtDisplay(w), XtWindow(w), im->bitmap,
-                                im->width, im->height);
+    switch (im->bpp) {
+      case 1: si = 0; ei = 2; break;
+      case 2: si = 0; ei = 4; break;
+      case 4: si = 4; ei = 20; break;
+    }
 
     /*
-     * Draw a rectangle and copy the image to the rectangle.
+     * Draw a rectangle around the image area.
      */
     XDrawRectangle(XtDisplay(w), XtWindow(w), gw->gedittb.gc, im->x, im->y,
                    im->width + 3, im->height + 3);
-    XCopyPlane(XtDisplay(w), gw->gedittb.image, XtWindow(w), gw->gedittb.gc,
-               0, 0, im->width, im->height, im->x + 2, im->y + 2, 1);
+
+    /*
+     * Go through and set all of the pixels with the correct colors
+     * for the image.
+     */
+    for (color = si; color < ei; color++) {
+        _XmuttGEditTBoxGetPixels(gw, im, color);
+        if (gw->gedittb.points_used > 0) {
+            if (im->bpp == 1) {
+                if (color)
+                  XSetForeground(XtDisplay(w), gw->gedittb.gc,
+                                 gw->primitive.foreground);
+                else
+                  XSetForeground(XtDisplay(w), gw->gedittb.gc,
+                                 gw->core.background_pixel);
+            } else
+              XSetForeground(XtDisplay(w), gw->gedittb.gc,
+                             gw->gedittb.colors[color - si]);
+            XDrawPoints(XtDisplay(w), XtWindow(w), gw->gedittb.gc,
+                        gw->gedittb.points, gw->gedittb.points_used,
+                        CoordModeOrigin);
+        }
+    }
+    XSetForeground(XtDisplay(w), gw->gedittb.gc,
+                   gw->primitive.foreground);
+
 }
 
 static void
@@ -785,6 +927,7 @@ XmuttGlyphEditToolboxWidget gw;
     int i, j, x, y, dx, dy;
     Dimension cp;
     _XmuttGEditTBoxButton *bp;
+    _XmuttGEditTBoxColor *cl;
     bdf_bitmap_t *im;
     XRectangle button;
     XPoint toggle[5];
@@ -928,6 +1071,32 @@ XmuttGlyphEditToolboxWidget gw;
             break;
         }
     }
+
+    /*
+     * Set up the color regions starting with the two
+     */
+    cl = gw->gedittb.cols;
+    button.width = gw->gedittb.cbw;
+    button.height = gw->gedittb.cbh;
+    for (i = 0; i < 16; i++) {
+        if (!i) {
+            button.x = cl[i].x = (_XmuttGEditTBoxPMargins(gw) >> 1) + 2;
+            button.y = cl[i].y = bp[_GETB_DRAW_TOGGLE].y + 2;
+        } else {
+            cl[i].x = button.x;
+            button.y = cl[i].y = cl[i - 1].y + gw->gedittb.cbh + 3;
+        }
+
+        if (cl[i].region == 0) {
+            cl[i].region = XCreateRegion();
+            XUnionRectWithRegion(&button, cl[i].region, cl[i].region);
+        } else {
+            dy = cl[i].y;
+            cl[i].y = button.y;
+            if ((dy = cl[i].y - dy) != 0)
+              XOffsetRegion(cl[i].region, cl[i].x, dy);
+        }
+    }
 }
 
 static void
@@ -967,8 +1136,9 @@ XmuttGlyphEditToolboxWidget gw;
 int x, y;
 #endif
 {
-    int i, which;
+    int i, which, top;
     _XmuttGEditTBoxButton *bp;
+    _XmuttGEditTBoxColor *cp;
     
     bp = gw->gedittb.buttons;
 
@@ -982,6 +1152,18 @@ int x, y;
           which = i;
     }
 
+    /*
+     * Check to see if the point is on one of the colors.
+     */
+    top = 0;
+    if (gw->gedittb.gimage && gw->gedittb.gimage->bpp > 1)
+      top = 1 << gw->gedittb.gimage->bpp;
+    cp = gw->gedittb.cols;
+    for (i = 0; which == -1 && i < top; i++) {
+        if (XPointInRegion(cp[i].region, x, y) == True)
+          which = i + _GETB_GLYPH_IMAGE + 1;
+    }
+
     return which;
 }
 
@@ -1025,7 +1207,7 @@ Cardinal *num_args;
     Screen *screen;
     Dimension wd, ht;
     Pixel unused;
-    int i, x, y, bpr, byte;
+    int i;
     _XmuttGEditTBoxButton *bp;
     bdf_bitmap_t *gimage;
 
@@ -1034,6 +1216,12 @@ Cardinal *num_args;
 
     gwc->gedittb_class.refcnt++;
 
+    /*
+     * Set the color spot size.
+     */
+    gw->gedittb.cbw = 20;
+    gw->gedittb.cbh = 10;
+
     /*
      * Get the color used to arm the buttons.
      */
@@ -1042,6 +1230,20 @@ Cardinal *num_args;
                 gw->core.background_pixel, &unused, &unused, &unused,
                 &gw->gedittb.arm_color);
 
+    /*
+     * Initialize the current color index.
+     */
+    if (gw->gedittb.gimage) {
+        if ((gw->gedittb.cidx = (1 << gw->gedittb.gimage->bpp) - 1) == 0)
+          gw->gedittb.cidx = -1;
+    } else
+      gw->gedittb.cidx = -1;
+
+    /*
+     * Initialize the points list.
+     */
+    gw->gedittb.points_size = gw->gedittb.points_used = 0;
+
     /*
      * Intialize the timer information.
      */
@@ -1119,15 +1321,23 @@ Cardinal *num_args;
     bp->help = help_strings[_GETB_GLYPH_IMAGE];
 
     /*
-     * Initialize the glyph image if it exists.
+     * Initialize the color structures.
+     */
+    for (i = 0; i < 16; i++) {
+        gw->gedittb.cols[i].x = gw->gedittb.cols[i].y = 0;
+        gw->gedittb.cols[i].region = 0;
+    }
+
+    /*
+     * Initialize the glyph image region.
      */
-    gw->gedittb.image = 0;
     gw->gedittb.image_region = 0;
     if (gw->gedittb.gimage != 0) {
         /*
          * Copy the image.
          */
         gimage = (bdf_bitmap_t *) XtMalloc(sizeof(bdf_bitmap_t));
+        gimage->bpp = gw->gedittb.gimage->bpp;
         gimage->x = gimage->y = 0;
         gimage->width = gw->gedittb.gimage->width;
         gimage->height = gw->gedittb.gimage->height;
@@ -1136,17 +1346,6 @@ Cardinal *num_args;
         (void) memcpy((char *) gimage->bitmap, gw->gedittb.gimage->bitmap,
                       gimage->bytes);
 
-        /*
-         * Shift the bits around so a pixmap can be made from the data.
-         */
-        bpr = (gimage->width + 7) >> 3;
-        for (y = 0; y < gimage->height; y++) {
-            for (x = 0; x < bpr; x++) {
-                byte = gimage->bitmap[(y * bpr) + x] & 0xff;
-                gimage->bitmap[(y * bpr) + x] = bitswap[byte];
-            }
-        }
-
         gw->gedittb.gimage = gimage;
     }
 
@@ -1246,13 +1445,6 @@ Widget w;
         bp[i].region = 0;
     }
 
-    /*
-     * Destroy the image pixmap if it exists.
-     */
-    if (gw->gedittb.image != 0)
-      XFreePixmap(XtDisplay(w), gw->gedittb.image);
-    gw->gedittb.image = 0;
-
     /*
      * Destroy the glyph image if it exists.
      */
@@ -1263,6 +1455,13 @@ Widget w;
     }
     gw->gedittb.gimage = 0;
 
+    /*
+     * Free the points if any have been allocated.
+     */
+    if (gw->gedittb.points_size > 0)
+      XtFree((char *) gw->gedittb.points);
+    gw->gedittb.points_used = gw->gedittb.points_size = 0;
+
     if (gwc->gedittb_class.refcnt > 0) {
         /*
          * Decrement the class reference count.
@@ -1343,6 +1542,8 @@ Region region;
         else
           _XmuttGEditTBoxDrawToggle(w, i);
     }
+
+    _XmuttGEditTBoxDrawColors(w);
 }
 
 static Boolean
@@ -1360,7 +1561,6 @@ Cardinal *num_args;
     Screen *screen;
     Pixel unused;
     Dimension width, height;
-    int x, y, bpr, byte;
     short ox, oy, owidth, oheight;
     bdf_bitmap_t *gimage;
     Boolean redo, repos;
@@ -1400,15 +1600,14 @@ Cardinal *num_args;
         repos = False;
 
         /*
-         * The glyph image has changed.  Delete the old one, free the
-         * image pixmap (if it has been created), and copy the new
-         * glyph image.
+         * The glyph image has changed.  Delete the old one, free the image
+         * pixmap (if it has been created), and copy the new glyph image.
          */
         ox = oy = owidth = oheight = 0;
         if (cgw->gedittb.gimage != 0) {
             /*
-             * Save the original width and height to determine
-             * if a reposition needs to be done.
+             * Save the original width and height to determine if a reposition
+             * needs to be done.
              */
             ox = cgw->gedittb.gimage->x;
             oy = cgw->gedittb.gimage->y;
@@ -1426,6 +1625,7 @@ Cardinal *num_args;
              * Copy the image.
              */
             gimage = (bdf_bitmap_t *) XtMalloc(sizeof(bdf_bitmap_t));
+            gimage->bpp = ngw->gedittb.gimage->bpp;
             gimage->width = ngw->gedittb.gimage->width;
             gimage->height = ngw->gedittb.gimage->height;
             gimage->bytes = ngw->gedittb.gimage->bytes;
@@ -1433,19 +1633,19 @@ Cardinal *num_args;
             (void) memcpy((char *) gimage->bitmap, ngw->gedittb.gimage->bitmap,
                           gimage->bytes);
 
-            /*
-             * Shift the bits around so a pixmap can be made from the data.
-             */
-            bpr = (gimage->width + 7) >> 3;
-            for (y = 0; y < gimage->height; y++) {
-                for (x = 0; x < bpr; x++) {
-                    byte = gimage->bitmap[(y * bpr) + x] & 0xff;
-                    gimage->bitmap[(y * bpr) + x] = bitswap[byte];
-                }
-            }
-
             ngw->gedittb.gimage = gimage;
 
+            /*
+             * Set the color index if necessary.
+             */
+            if (ngw->gedittb.cidx == -1) {
+                if ((ngw->gedittb.cidx = (1 << gimage->bpp) - 1) == 0)
+                  ngw->gedittb.cidx = 1;
+            } else if (gimage->bpp == 1)
+              ngw->gedittb.cidx = 1;
+            else if (ngw->gedittb.cidx >= (1 << gimage->bpp))
+              ngw->gedittb.cidx = (1 << gimage->bpp) - 1;
+
             if (gimage->width != owidth || gimage->height != oheight) {
                 /*
                  * Since a size change is taking place, figure out
@@ -1474,10 +1674,6 @@ Cardinal *num_args;
         } else if (owidth || oheight)
           repos = True;
 
-        if (cgw->gedittb.image != 0)
-          XFreePixmap(XtDisplay(curr), cgw->gedittb.image);
-        cgw->gedittb.image = ngw->gedittb.image = 0;
-
         /*
          * If the widget will not be redrawn and the new glyph image didn't
          * change size, then simply redraw the glyph image.
@@ -1587,6 +1783,20 @@ Widget w;
     return (which >= 0) ? cback_values[which] : which;
 }
 
+void
+#ifndef _NO_PROTO
+XmuttGlyphEditToolboxSetColor(Widget w, int color)
+#else
+XmuttGlyphEditToolboxSetColor(w, color)
+Widget w;
+int color;
+#endif
+{
+    _XmuttGEditTBoxCheckClass(w);
+
+    _XmuttGEditTBoxSelectColor(w, color);
+}
+
 /**************************************************************************
  *
  * Action functions.
@@ -1704,9 +1914,21 @@ Cardinal *num_params;
 
     b = _XmuttGEditTBoxXYToButton(gw, event->xbutton.x, event->xbutton.y);
 
-    if (b < 0 || b > _GETB_DRIGHT_BUTTON)
+    if (b < 0 || b == _GETB_GLYPH_IMAGE)
       return;
 
+    /*
+     * Handle the case of a color being selected first.
+     */
+    if (b > _GETB_GLYPH_IMAGE) {
+        _XmuttGEditTBoxSelectColor(w, b - (_GETB_GLYPH_IMAGE + 1));
+        cb.reason = XmuttGLYPHEDIT_COLOR;
+        cb.event = event;
+        cb.color = b - (_GETB_GLYPH_IMAGE + 1);
+        XtCallCallbackList(w, gw->gedittb.activate, (XtPointer) &cb);
+        return;
+    }
+
     if (bp[b].toggle == True && bp[b].set == False) {
         for (i = 0; i < 2; i++) {
             o = bp[b].other_toggles[i];
@@ -1815,9 +2037,10 @@ String *params;
 Cardinal *num_params;
 #endif
 {
+    int b;
     XmuttGlyphEditToolboxWidget gw;
     _XmuttGEditTBoxButton *bp;
-    int b;
+    char cbuf[16];
     XmuttGlyphEditToolboxCallbackStruct cb;
 
     gw = (XmuttGlyphEditToolboxWidget) w;
@@ -1842,7 +2065,11 @@ Cardinal *num_params;
     if (b != gw->gedittb.last) {
         cb.reason = XmuttGLYPHEDIT_CONTEXT_HELP;
         cb.event = event;
-        cb.help = bp[b].help;
+        if (b > _GETB_GLYPH_IMAGE) {
+            sprintf(cbuf, "Color %d", b - (_GETB_GLYPH_IMAGE + 1));
+            cb.help = cbuf;
+        } else
+          cb.help = bp[b].help;
         XtCallCallbackList(w, gw->gedittb.help, (XtPointer) &cb);
     }
     gw->gedittb.last = b;
@@ -1946,6 +2173,15 @@ static XtResource resources[] = {
         XmRImmediate,
         (XtPointer) 0,
     },
+    {
+        XmNcolorList,
+        XmCColorList,
+        XmRPointer,
+        sizeof(unsigned long *),
+        XtOffset(XmuttGlyphEditToolboxWidget, gedittb.colors),
+        XmRImmediate,
+        (XtPointer) 0,
+    },
 };
 
 externaldef(xmuttglyphedittboxclassrec)
