@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,11 +24,11 @@
 #define _h_XmuttGEditToolbox
 
 /*
- * $Id: GEditTB.h,v 1.1 1998/07/24 16:12:08 mleisher Exp $
+ * $Id: GEditTB.h,v 1.3 1999/05/03 17:17:49 mleisher Exp $
  */
 
 #include <Xm/Xm.h>
-#include "bdf.h"
+#include "bdfP.h"
 #include "GEShared.h"
 
 /*
@@ -64,12 +64,16 @@ XtIsSubclass(w, xmuttGlyphEditToolboxWidgetClass)
 #define XmNcontextHelpCallback "contextHelpCallback"
 #define XmCContextHelpCallback "ContextHelpCallback"
 
+#define XmNcolorList "colorList"
+#define XmCColorList "ColorList"
+
 /*
  * Values that can appear in the `reason' field of the callback structure.
  */
 #define XmuttGLYPHEDIT_CONTEXT_HELP -1
 #define XmuttGLYPHEDIT_OPERATION    -2
 #define XmuttGLYPHEDIT_ACTION       -3
+#define XmuttGLYPHEDIT_COLOR        -4
 
 /*
  * Callback structure used for XmNcontextHelpCallback and XmNactivateCallback.
@@ -81,6 +85,7 @@ typedef struct {
     XEvent *event;
     char *help;
     int op_action;
+    int color;
 } XmuttGlyphEditToolboxCallbackStruct;
 
 extern void XmuttGlyphEditToolboxSetToggle __((Widget w, int toggle,
@@ -88,6 +93,8 @@ extern void XmuttGlyphEditToolboxSetToggle __((Widget w, int toggle,
 
 extern int XmuttGlyphEditToolboxQueryToggle __((Widget w));
 
+extern void XmuttGlyphEditToolboxSetColor __((Widget w, int color));
+
 #ifdef __cplusplus
 }
 #endif
