@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,14 +24,14 @@
 #define _h_XmuttGEditToolboxP
 
 /*
- * $Id: GEditTBP.h,v 1.1 1998/07/24 16:12:09 mleisher Exp $
+ * $Id: GEditTBP.h,v 1.4 1999/05/03 17:17:49 mleisher Exp $
  */
 
 #include <stdio.h>
 #include <Xm/XmP.h>
 #if (XmVERSION == 1) && (XmREVISION >= 2)
 #include <Xm/PrimitiveP.h>
-#else if (XmVERSION == 2)
+#elif (XmVERSION == 2)
 #include <X11/CoreP.h>
 #include <Xm/PrimitiveP.h>
 #endif
@@ -56,6 +56,12 @@ typedef struct {
     Boolean toggle;
 } _XmuttGEditTBoxButton;
 
+typedef struct {
+    int x;
+    int y;
+    Region region;
+} _XmuttGEditTBoxColor;
+
 /*
  * Class and instance structures.
  */
@@ -96,6 +102,8 @@ typedef struct {
 
     bdf_bitmap_t *gimage;
 
+    unsigned long *colors;
+
     /*
      * Private members.
      */
@@ -106,18 +114,30 @@ typedef struct {
     GC tsgc;
     GC bsgc;
 
+    int cidx;
+
     XtIntervalId timer;
     int timer_button;
     int timer_count;
 
     Pixel arm_color;
 
-    Pixmap image;
     Region image_region;
 
     int pressed;
     int last;
 
+    XPoint *points;
+    unsigned long points_size;
+    unsigned long points_used;
+
+    /*
+     * Width and height of the color spots.
+     */
+    int cbw;
+    int cbh;
+
+    _XmuttGEditTBoxColor cols[16];
     _XmuttGEditTBoxButton buttons[18];
 
     Boolean hasfocus;
