@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: GTest.c,v 1.1 1998/07/24 16:12:10 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: GTest.c,v 1.6 1999/05/03 17:17:49 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: GTest.c,v 1.1 1998/07/24 16:12:10 mleisher Exp $";
+static char rcsid[] = "$Id: GTest.c,v 1.6 1999/05/03 17:17:49 mleisher Exp $";
 #endif
 #endif
 
@@ -257,7 +257,11 @@ XmuttGlyphTestWidget gw;
         bbx.ascent = MAX(bbx.ascent, lp->glyphs[i].font->bbx.ascent);
         bbx.descent = MAX(bbx.descent, lp->glyphs[i].font->bbx.descent);
         bbx.width = MAX(bbx.width, lp->glyphs[i].font->bbx.width);
-        wd += lp->glyphs[i].glyph->bbx.x_offset +
+        if (lp->glyphs[i].glyph->dwidth !=
+            lp->glyphs[i].glyph->bbx.width + lp->glyphs[i].glyph->bbx.x_offset)
+          wd += lp->glyphs[i].glyph->dwidth;
+        else
+          wd += lp->glyphs[i].glyph->bbx.x_offset +
             lp->glyphs[i].glyph->bbx.width;
     }
 
@@ -277,6 +281,7 @@ XmuttGlyphTestWidget gw;
      */
     if (wd != lp->width) {
         lp->width = wd;
+
         /*
          * If the line width overflows the window width, set the changed flag.
          */
@@ -350,64 +355,73 @@ Boolean force;
     }
 }
 
-/*
- * Generate a list of points that represents the image bitmap relative
- * to the baseline already set.
- */
 static void
 #ifndef _NO_PROTO
-_XmuttGlyphTestGlyphImage(XmuttGlyphTestWidget gw, bdf_glyph_t *glyph,
-                          Position x, Position y)
+_XmuttGlyphTestGetPixels(XmuttGlyphTestWidget gw, bdf_glyph_t *glyph,
+                         bdf_font_t *font, Position x, Position y, int color)
 #else
-_XmuttGlyphTestGlyphImage(gw, glyph, x, y)
+_XmuttGlyphTestGetPixels(gw, glyph, font, x, y, color)
 XmuttGlyphTestWidget gw;
 bdf_glyph_t *glyph;
+bdf_font_t *font;
 Position x, y;
+int color;
 #endif
 {
-    int i, j, bpr, points;
-    unsigned char *bmap;
-    XPoint *pp;
+    int byte;
+    unsigned short i, j, bpr, si, di, nx;
+    unsigned char *masks;
 
     gw->gtest.image_used = 0;
-    bmap = glyph->bitmap;
-    bpr = (glyph->bbx.width + 7) >> 3;
 
-    /*
-     * Resize the point list to be the area of the glyph.
-     */
-    points = glyph->bbx.width * glyph->bbx.height;
-    if (points > gw->gtest.image_size) {
-        if (gw->gtest.image_size == 0)
-          gw->gtest.image = (XPoint *) XtMalloc(sizeof(XPoint) * points);
-        else
-          gw->gtest.image = (XPoint *)
-              XtRealloc((char *) gw->gtest.image, sizeof(XPoint) * points);
-        gw->gtest.image_size = points;
+    switch (font->bpp) {
+      case 1: masks = onebpp; di = 7; break;
+      case 2: masks = twobpp; di = 3; break;
+      case 4: masks = fourbpp; di = 1; break;
     }
 
-    pp = gw->gtest.image;
+    bpr = ((glyph->bbx.width * font->bpp) + 7) >> 3;
     for (i = 0; i < glyph->bbx.height; i++) {
-        for (j = 0; j < glyph->bbx.width; j++) {
-            if (bmap[(i * bpr) + (j >> 3)] & (0x80 >> (j & 7))) {
-                pp->x = x + glyph->bbx.x_offset + j;
-                pp->y = (y - glyph->bbx.ascent) + i;
-                pp++;
+        for (nx = j = 0; j < glyph->bbx.width; j++, nx += font->bpp) {
+            si = (nx & 7) / font->bpp;
+
+            byte = glyph->bitmap[(i * bpr) + (nx >> 3)] & masks[si];
+            if (di > si)
+              byte >>= (di - si) * font->bpp;
+            if (byte == color) {
+                if (gw->gtest.image_used == gw->gtest.image_size) {
+                    if (gw->gtest.image_size == 0)
+                      gw->gtest.image =
+                          (XPoint *) XtMalloc(sizeof(XPoint) * 64);
+                    else
+                      gw->gtest.image = (XPoint *)
+                          XtRealloc((char *) gw->gtest.image,
+                                    sizeof(XPoint) *
+                                    (gw->gtest.image_size + 64));;
+                    gw->gtest.image_size += 64;
+                }
+                gw->gtest.image[gw->gtest.image_used].x =
+                    x + glyph->bbx.x_offset + j;
+                gw->gtest.image[gw->gtest.image_used].y =
+                    (y - glyph->bbx.ascent) + i;
+                gw->gtest.image_used++;
             }
         }
     }
-    gw->gtest.image_used = pp - gw->gtest.image;
 }
 
 static void
 #ifndef _NO_PROTO
-_XmuttGlyphTestDrawGlyph(Widget w, bdf_glyph_t *glyph)
+_XmuttGlyphTestDrawGlyph(Widget w, bdf_glyph_t *glyph, bdf_font_t *font)
 #else
-_XmuttGlyphTestDrawGlyph(w, glyph)
+_XmuttGlyphTestDrawGlyph(w, glyph, font)
 Widget w;
 bdf_glyph_t *glyph;
+bdf_font_t *font;
 #endif
 {
+    int i, s, e;
+    Position rx, ry;
     XmuttGlyphTestWidget gw;
 
     if (!XtIsRealized(w))
@@ -415,17 +429,45 @@ bdf_glyph_t *glyph;
 
     gw = (XmuttGlyphTestWidget) w;
 
-    if (gw->gtest.dir == XmuttGlyphTestLeftToRight)
-      _XmuttGlyphTestGlyphImage(gw, glyph,
-                                gw->gtest.line.cpoint.x,
-                                gw->gtest.line.cpoint.y);
-    else
-      _XmuttGlyphTestGlyphImage(gw, glyph,
-                                gw->gtest.line.cpoint.x - glyph->bbx.width,
-                                gw->gtest.line.cpoint.y);
+    switch (font->bpp) {
+      case 1: s = 0; e = 2; break;
+      case 2: s = 0; e = 4; break;
+      case 4: s = 4; e = 20; break;
+    }
 
-    XDrawPoints(XtDisplay(w), XtWindow(w), gw->gtest.glyphgc,
-                gw->gtest.image, gw->gtest.image_used, CoordModeOrigin);
+    ry = gw->gtest.line.cpoint.y;
+    rx = gw->gtest.line.cpoint.x;
+    if (gw->gtest.dir != XmuttGlyphTestLeftToRight)
+      rx -= glyph->bbx.width;
+
+    for (i = s; i < e; i++) {
+        if (i == 0)
+          /*
+           * Avoid drawing the pixels with the background color.
+           */
+          continue;
+
+        _XmuttGlyphTestGetPixels(gw, glyph, font, rx, ry, i);
+        if (gw->gtest.image_used > 0) {
+            if (font->bpp > 1) {
+                if (i - s)
+                  XSetForeground(XtDisplay(w), gw->gtest.glyphgc,
+                                 gw->gtest.colors[i - s]);
+                else
+                  /*
+                   * Make all 0 colors the same as the background.
+                   */
+                  XSetForeground(XtDisplay(w), gw->gtest.glyphgc,
+                                 gw->core.background_pixel);
+            }
+            XDrawPoints(XtDisplay(w), XtWindow(w), gw->gtest.glyphgc,
+                        gw->gtest.image, gw->gtest.image_used,
+                        CoordModeOrigin);
+        }
+    }
+    if (font->bpp > 1)
+      XSetForeground(XtDisplay(w), gw->gtest.glyphgc,
+                     gw->primitive.foreground);
 }
 
 static void
@@ -469,13 +511,26 @@ Widget w;
                 gp->glyph->bbx.x_offset > gp->glyph->bbx.width)
               lp->cpoint.x -= gp->glyph->bbx.width - gp->glyph->bbx.x_offset;
         }
-        _XmuttGlyphTestDrawGlyph(w, gp->glyph);
+        _XmuttGlyphTestDrawGlyph(w, gp->glyph, gp->font);
 
-        if (gw->gtest.dir == XmuttGlyphTestLeftToRight)
-          lp->cpoint.x += gp->glyph->bbx.width + gp->glyph->bbx.x_offset;
+        if (gw->gtest.dir == XmuttGlyphTestLeftToRight) {
+            if (gp->glyph->dwidth !=
+                gp->glyph->bbx.width + gp->glyph->bbx.x_offset)
+              lp->cpoint.x += gp->glyph->dwidth;
+            else
+              lp->cpoint.x += gp->glyph->bbx.width + gp->glyph->bbx.x_offset;
+        } else {
+            if (gp->glyph->dwidth !=
+                gp->glyph->bbx.width + gp->glyph->bbx.x_offset)
+              lp->cpoint.x -= gp->glyph->dwidth;
+            else
+              lp->cpoint.x -= gp->glyph->bbx.width - gp->glyph->bbx.x_offset;
+        }
+        if (gp->glyph->dwidth !=
+            gp->glyph->bbx.width + gp->glyph->bbx.x_offset)
+          lp->width += gp->glyph->dwidth;
         else
-          lp->cpoint.x -= gp->glyph->bbx.width - gp->glyph->bbx.x_offset;
-        lp->width += gp->glyph->bbx.width + gp->glyph->bbx.x_offset;
+          lp->width += gp->glyph->bbx.width + gp->glyph->bbx.x_offset;
     }
 }
 
@@ -773,14 +828,26 @@ bdf_glyph_t *glyph;
               lp->cpoint.x -= glyph->bbx.width - glyph->bbx.x_offset;
         }
 
-        _XmuttGlyphTestDrawGlyph(w, glyph);
+        _XmuttGlyphTestDrawGlyph(w, glyph, font);
 
-        if (gw->gtest.dir == XmuttGlyphTestLeftToRight)
-          lp->cpoint.x += glyph->bbx.width + glyph->bbx.x_offset;
+        if (gw->gtest.dir == XmuttGlyphTestLeftToRight) {
+            if (glyph->dwidth !=
+                glyph->bbx.width + glyph->bbx.x_offset)
+              lp->cpoint.x += glyph->dwidth;
+            else
+              lp->cpoint.x += glyph->bbx.width + glyph->bbx.x_offset;
+        } else {
+            if (glyph->dwidth !=
+                glyph->bbx.width + glyph->bbx.x_offset)
+              lp->cpoint.x -= glyph->dwidth;
+            else
+              lp->cpoint.x -= glyph->bbx.width - glyph->bbx.x_offset;
+        }
+        if (glyph->dwidth !=
+            glyph->bbx.width + glyph->bbx.x_offset)
+          lp->width += glyph->dwidth;
         else
-          lp->cpoint.x -= glyph->bbx.width - glyph->bbx.x_offset;
-
-        lp->width += glyph->bbx.width + glyph->bbx.x_offset;
+          lp->width += glyph->bbx.width + glyph->bbx.x_offset;
     }
 
     /*
@@ -1069,6 +1136,15 @@ static XtResource resources[] = {
         XmRImmediate,
         (XtPointer) 0,
     },
+    {
+        XmNcolorList,
+        XmCColorList,
+        XmRPointer,
+        sizeof(unsigned long *),
+        XtOffset(XmuttGlyphTestWidget, gtest.colors),
+        XmRImmediate,
+        (XtPointer) 0,
+    },
 };
 
 externaldef(xmuttglyphtestclassrec)
