@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@
 #define _h_XmuttGlyphTest
 
 /*
- * $Id: GTest.h,v 1.1 1998/07/24 16:12:10 mleisher Exp $
+ * $Id: GTest.h,v 1.3 1999/05/03 17:17:50 mleisher Exp $
  */
 #include <Xm/Xm.h>
 #include "bdfP.h"
@@ -73,6 +73,9 @@ typedef struct _XmuttGlyphTestRec      *XmuttGlyphTestWidget;
 #define XmNaddGlyphCallback "addGlyphCallback"
 #define XmCAddGlyphCallback "AddGlyphCallback"
 
+#define XmNcolorList "colorList"
+#define XmCColorList "ColorList"
+
 extern void XmuttGlyphTestAddGlyph __((Widget w, bdf_font_t *font,
                                        bdf_glyph_t *glyph));
 
