@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,46 +22,14 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: ProgBar.c,v 1.1 1998/07/24 16:12:13 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: ProgBar.c,v 1.4 1999/10/23 13:43:35 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: ProgBar.c,v 1.1 1998/07/24 16:12:13 mleisher Exp $";
+static char rcsid[] = "$Id: ProgBar.c,v 1.4 1999/10/23 13:43:35 mleisher Exp $";
 #endif
 #endif
 
 #include "ProgBarP.h"
 
-#if 0
-static void
-#ifndef _NO_PROTO
-_XmuttProgressBarAttemptResize(XmuttProgressBarWidget pw, Dimension width,
-                               Dimension height)
-#else
-_XmuttProgressBarAttemptResize(pw, width, height)
-XmuttProgressBarWidget pw;
-Dimension width, height;
-#endif
-{
-    Dimension rwidth, rheight, owidth, oheight;
-    XtGeometryResult res;
-
-    owidth = pw->core.width;
-    oheight = pw->core.height;
-
-    res = XtMakeResizeRequest((Widget) pw, width, height, &rwidth, &rheight);
-
-    if (res == XtGeometryAlmost) {
-        res = XtMakeResizeRequest((Widget) pw, rwidth, rheight,
-                                  &rwidth, &rheight);
-        if (rwidth == owidth && rheight == oheight)
-          res = XtGeometryNo;
-    } else if (pw->core.width != width && pw->core.height != height)
-      res = XtGeometryNo;
-
-    if (res == XtGeometryYes)
-      (*pw->core.widget_class->core_class.resize)((Widget) pw);
-}
-#endif
-
 static Dimension
 #ifndef _NO_PROTO
 _XmuttProgressBarPMargins(XmuttProgressBarWidget pw)
@@ -464,11 +432,12 @@ Widget w;
                           &pw->progress.bar);
         XmStringFree(ns);
 
-        /*
-         * And redraw draw the progress bar itself.
-         */
-        XFillRectangle(XtDisplay(w), XtWindow(w), pw->progress.bargc,
-                       bar.x, bar.y, bar.width, bar.height);
+        if (pw->progress.number_only == False)
+          /*
+           * And redraw draw the progress bar itself.
+           */
+          XFillRectangle(XtDisplay(w), XtWindow(w), pw->progress.bargc,
+                         bar.x, bar.y, bar.width, bar.height);
         pw->progress.lastwidth = bar.width;
     }
 }
@@ -537,7 +506,8 @@ Widget w;
                           nswd, XmALIGNMENT_BEGINNING,
                           XmSTRING_DIRECTION_L_TO_R,
                           &pw->progress.bar);
-        if (bar.x + bar.width > nsx && bar.x + pw->progress.lastwidth >= nsx) {
+        if (pw->progress.number_only == False &&
+            bar.x + bar.width > nsx && bar.x + pw->progress.lastwidth >= nsx) {
             if (bar.x + pw->progress.lastwidth >= nsx + nswd)
               XFillRectangle(XtDisplay(w), XtWindow(w), pw->progress.bargc,
                              nsx, bar.y, nswd, bar.height);
@@ -557,7 +527,8 @@ Widget w;
     /*
      * Add check to see if new value is smaller or larger.
      */
-    if (bar.width != pw->progress.lastwidth)
+    if (pw->progress.number_only == False &&
+        bar.width != pw->progress.lastwidth)
       XFillRectangle(XtDisplay(w), XtWindow(w), pw->progress.bargc,
                      bar.x + pw->progress.lastwidth, bar.y,
                      bar.width - pw->progress.lastwidth, bar.height);
@@ -896,7 +867,16 @@ static XtResource resources[] = {
         XtOffset(XmuttProgressBarWidget, progress.dir),
         XmRImmediate,
         (XtPointer) XmuttProgressBarLeftToRight,
-    }
+    },
+    {
+        XmNpercentOnly,
+        XmCPercentOnly,
+        XmRBoolean,
+        sizeof(Boolean),
+        XtOffset(XmuttProgressBarWidget, progress.number_only),
+        XmRImmediate,
+        (XtPointer) False,
+    },
 };
 
 externaldef(xmuttprogbarclassrec) XmuttProgBarClassRec xmuttProgBarClassRec = {
