@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@
 #define _h_ProgBar
 
 /*
- * $Id: ProgBar.h,v 1.1 1998/07/24 16:12:14 mleisher Exp $
+ * $Id: ProgBar.h,v 1.3 1999/05/03 17:17:50 mleisher Exp $
  */
 
 #include <Xm/Xm.h>
@@ -69,6 +69,9 @@ extern "C" {
 #define XmNupdateDirection "updateDirection"
 #define XmCUpdateDirection "UpdateDirection"
 
+#define XmNpercentOnly "percentOnly"
+#define XmCPercentOnly "PercentOnly"
+
 #define XmuttProgressBarLeftToRight 0
 #define XmuttProgressBarRightToLeft 1
 
