@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,14 +24,14 @@
 #define _h_ProgBarP
 
 /*
- * $Id: ProgBarP.h,v 1.1 1998/07/24 16:12:14 mleisher Exp $
+ * $Id: ProgBarP.h,v 1.4 1999/05/03 17:17:51 mleisher Exp $
  */
 
 #include <stdio.h>
 #include <Xm/XmP.h>
 #if (XmVERSION == 1) && (XmREVISION >= 2)
 #include <Xm/PrimitiveP.h>
-#else if (XmVERSION == 2)
+#elif (XmVERSION == 2)
 #include <X11/CoreP.h>
 #include <Xm/PrimitiveP.h>
 #endif
@@ -93,6 +93,8 @@ typedef struct {
      */
     int dir;
 
+    Boolean number_only;
+
     /*
      * Private members.
      */
