@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@
 #define _h_bdf
 
 /*
- * $Id: bdf.h,v 1.1 1998/07/24 16:12:17 mleisher Exp $
+ * $Id: bdf.h,v 1.15 1999/10/23 13:43:36 mleisher Exp $
  */
 
 #include <stdio.h>
@@ -62,14 +62,18 @@ extern "C" {
  *
  **************************************************************************/
 
+#define BDF_UNIX_EOL 1           /* Save fonts with Unix LF.              */
+#define BDF_DOS_EOL  2           /* Save fonts with DOS CRLF.             */
+#define BDF_MAC_EOL  3           /* Save fonts with Mac CR.               */
+
 #define BDF_CORRECT_METRICS 0x01 /* Correct invalid metrics when loading. */
 #define BDF_KEEP_COMMENTS   0x02 /* Preserve the font comments.           */
 #define BDF_KEEP_UNENCODED  0x04 /* Keep the unencoded glyphs.            */
 #define BDF_PROPORTIONAL    0x08 /* Font has proportional spacing.        */
-#define BDF_MONOSPACE       0x10 /* Font has mono spacing.                */
+#define BDF_MONOWIDTH       0x10 /* Font has mono width.                  */
 #define BDF_CHARCELL        0x20 /* Font has charcell spacing.            */
 
-#define BDF_ALL_SPACING (BDF_PROPORTIONAL|BDF_MONOSPACE|BDF_CHARCELL)
+#define BDF_ALL_SPACING (BDF_PROPORTIONAL|BDF_MONOWIDTH|BDF_CHARCELL)
 
 #define BDF_DEFAULT_LOAD_OPTIONS \
     (BDF_CORRECT_METRICS|BDF_KEEP_COMMENTS|BDF_KEEP_UNENCODED|BDF_PROPORTIONAL)
@@ -84,6 +88,8 @@ typedef struct {
     long point_size;
     unsigned long resolution_x;
     unsigned long resolution_y;
+    int bits_per_pixel;
+    int eol;
 } bdf_options_t;
 
 /*
@@ -119,6 +125,40 @@ typedef struct {
     } value;            /* Value of the property.                       */
 } bdf_property_t;
 
+/**************************************************************************
+ *
+ * SBIT metrics specific structures.
+ *
+ **************************************************************************/
+
+/*
+ * Boolean flags for SBIT metrics files.
+ */
+#define BDF_SBIT_MONO_ADVANCE 0x0001
+#define BDF_SBIT_ADD_EBLC     0x0002
+#define BDF_SBIT_APPLE_COMPAT 0x0004
+
+/*
+ * Direction macros (inclusive, can be combined).
+ */
+#define BDF_SBIT_HORIZONTAL   0x0008
+#define BDF_SBIT_VERTICAL     0x0010
+
+/*
+ * Bitmap storage options (exclusive, cannot be combined).
+ */
+#define BDF_SBIT_STORE_SMALL  0x0020
+#define BDF_SBIT_STORE_FAST   0x0040
+
+typedef struct {
+    short cnum;         /* Caret slope numerator.                       */
+    short cdenom;       /* Caret slope denominator.                     */
+    short coff;         /* Caret offset.                                */
+    short sx;           /* Scaled version horizontal PPEM size.         */
+    short sy;           /* Scaled version vertical PPEM size (optional).*/
+    short flags;        /* Booleans and other non-numeric values.       */
+} bdf_sbit_t;
+
 /**************************************************************************
  *
  * BDF font metric and glyph types.
@@ -133,6 +173,8 @@ typedef struct {
     short y;
     unsigned short width;
     unsigned short height;
+    unsigned short bpp;
+    unsigned short pad;
     unsigned char *bitmap;
     unsigned long bytes;
 } bdf_bitmap_t;
@@ -169,6 +211,8 @@ typedef struct {
 } bdf_glyph_t;
 
 typedef struct {
+    unsigned short pad;         /* Pad to 4-byte boundary.              */
+    unsigned short bpp;         /* Bits per pixel.                      */
     long start;                 /* Beginning encoding value of glyphs.  */
     long end;                   /* Ending encoding value of glyphs.     */
     bdf_glyph_t *glyphs;        /* Glyphs themselves.                   */
@@ -189,7 +233,7 @@ typedef struct {
 
     int spacing;                /* Font spacing value.                   */
 
-    unsigned short monowidth;   /* Logical width for monospace font.     */
+    unsigned short monowidth;   /* Logical width for monowidth font.     */
 
     long default_glyph;         /* Encoding of the default glyph.        */
 
@@ -221,7 +265,12 @@ typedef struct {
     unsigned long nmod[2048];   /* Bitmap indicating modified glyphs.    */
     unsigned long umod[2048];   /* Bitmap indicating modified unencoded. */
 
-    int modified;               /* Boolean indicating font modified.     */
+    unsigned short modified;    /* Boolean indicating font modified.     */
+    unsigned short bpp;         /* Bits per pixel.                       */
+
+    bdf_sbit_t *sbits;          /* Associcated SBIT metrics.             */
+    unsigned long sbits_used;   /* Number of SBIT metrics entries.       */
+    unsigned long sbits_size;   /* Amount of entries allocated.          */
 } bdf_font_t;
 
 /**************************************************************************
@@ -233,7 +282,8 @@ typedef struct {
 typedef struct {
     char *name;
     long encoding;              /* The glyph encoding.                  */
-    int unencoded;              /* Whether the glyph was unencoded.     */
+    unsigned short unencoded;   /* Whether the glyph was unencoded.     */
+    unsigned short bpp;         /* Bits per pixel.                      */
     int spacing;                /* Font spacing.                        */
     long resolution_x;          /* Horizontal resolution.               */
     long resolution_y;          /* Vertical resolution.                 */
@@ -251,6 +301,8 @@ typedef struct {
     short glyph_x;              /* Top-left X position of glyph.        */
     short glyph_y;              /* Top-left Y position of glyph.        */
     unsigned short modified;    /* Flag indicating if bitmap modified.  */
+    short cap_height;           /* Font CAP_HEIGHT if it exists.        */
+    short x_height;             /* Font X_HEIGHT if it exists.          */
     bdf_bitmap_t sel;           /* Selected portion of the glyph bitmap.*/
 } bdf_glyph_grid_t;
 
@@ -263,18 +315,24 @@ typedef struct {
 /*
  * Callback reasons.
  */
-#define BDF_LOAD_START      1
-#define BDF_LOADING         2
-#define BDF_SAVE_START      3
-#define BDF_SAVING          4
-#define BDF_TRANSLATE_START 5
-#define BDF_TRANSLATING     6
-#define BDF_ROTATE_START    7
-#define BDF_ROTATING        8
-#define BDF_SHEAR_START     9
-#define BDF_SHEARING        10
-#define BDF_WARNING         20
-#define BDF_ERROR           21
+#define BDF_LOAD_START       1
+#define BDF_LOADING          2
+#define BDF_SAVE_START       3
+#define BDF_SAVING           4
+#define BDF_TRANSLATE_START  5
+#define BDF_TRANSLATING      6
+#define BDF_ROTATE_START     7
+#define BDF_ROTATING         8
+#define BDF_SHEAR_START      9
+#define BDF_SHEARING         10
+#define BDF_GLYPH_NAME_START 11
+#define BDF_GLYPH_NAME       12
+#define BDF_EXPORT_START     13
+#define BDF_EXPORTING        14
+#define BDF_EMBOLDEN_START   15
+#define BDF_EMBOLDENING      16
+#define BDF_WARNING          20
+#define BDF_ERROR            21
 
 /*
  * Error codes.
@@ -335,7 +393,7 @@ extern void bdf_default_options __((bdf_options_t *opts));
  */
 extern bdf_font_t *bdf_new_font __((char *name, long point_size,
                                     long resolution_x, long resolution_y,
-                                    long spacing));
+                                    long spacing, int bpp));
 extern bdf_font_t *bdf_load_font __((FILE *in, bdf_options_t *opts,
                                      bdf_callback_t callback, void *data));
 extern bdf_font_t *bdf_load_hbf_font __((char *filename, bdf_options_t *opts,
@@ -360,6 +418,12 @@ extern void bdf_save_font __((FILE *out, bdf_font_t *font,
                               bdf_options_t *opts, bdf_callback_t callback,
                               void *data));
 
+extern void bdf_save_sbit_metrics __((FILE *out, bdf_font_t *font,
+                                      bdf_options_t *opts, char *appname));
+
+extern void bdf_export_hex __((FILE *out, bdf_font_t *font,
+                               bdf_callback_t callback, void *data));
+
 extern int bdf_export_psf __((FILE *out, bdf_font_t *font));
 
 extern void bdf_free_font __((bdf_font_t *font));
@@ -454,6 +518,7 @@ extern void bdf_set_default_metrics __((bdf_font_t *font));
  */
 extern int bdf_glyph_modified __((bdf_font_t *font, long which,
                                   int unencoded));
+
 extern void bdf_copy_glyphs __((bdf_font_t *font, long start, long end,
                                 bdf_glyphlist_t *glyphs, int unencoded));
 
@@ -466,8 +531,8 @@ extern int bdf_insert_glyphs __((bdf_font_t *font, long start,
 extern int bdf_replace_glyphs __((bdf_font_t *font, long start,
                                   bdf_glyphlist_t *glyphs, int unencoded));
 
-extern int bdf_update_font __((bdf_font_t *font, bdf_glyph_t *glyph,
-                               int unencoded));
+extern int bdf_merge_glyphs __((bdf_font_t *font, long start,
+                                bdf_glyphlist_t *glyphs, int unencoded));
 
 /**************************************************************************
  *
@@ -490,6 +555,15 @@ extern void bdf_update_properties_from_name __((bdf_font_t *font));
 
 extern int bdf_update_average_width __((bdf_font_t *font));
 
+extern int bdf_set_unicode_glyph_names __((FILE *in, bdf_font_t *font,
+                                           bdf_callback_t callback));
+
+extern int bdf_set_adobe_glyph_names __((FILE *in, bdf_font_t *font,
+                                         bdf_callback_t callback));
+
+extern int bdf_set_glyph_code_names __((int prefix, bdf_font_t *font,
+                                        bdf_callback_t callback));
+
 /**************************************************************************
  *
  * Glyph grid API.
@@ -520,10 +594,11 @@ extern int bdf_grid_resize __((bdf_glyph_grid_t *grid,
                                bdf_metrics_t *metrics));
 extern int bdf_grid_crop __((bdf_glyph_grid_t *grid, int grid_modified));
 
-extern int bdf_grid_set_pixel __((bdf_glyph_grid_t *grid, short x, short y));
+extern int bdf_grid_set_pixel __((bdf_glyph_grid_t *grid, short x, short y,
+                                  int val));
 extern int bdf_grid_clear_pixel __((bdf_glyph_grid_t *grid, short x, short y));
 extern int bdf_grid_invert_pixel __((bdf_glyph_grid_t *grid,
-                                     short x, short y));
+                                     short x, short y, int val));
 extern int bdf_grid_shift __((bdf_glyph_grid_t *grid, short xcount,
                               short ycount));
 extern int bdf_grid_flip __((bdf_glyph_grid_t *grid, short dir));
@@ -531,6 +606,7 @@ extern int bdf_grid_rotate __((bdf_glyph_grid_t *grid, short degrees,
                                int *resize));
 extern int bdf_grid_shear __((bdf_glyph_grid_t *grid, short degrees,
                               int *resize));
+extern int bdf_grid_embolden __((bdf_glyph_grid_t *grid));
 
 /*
  * Glyph grid selection functions.
@@ -547,6 +623,11 @@ extern int bdf_in_selection __((bdf_glyph_grid_t *grid, short x, short y,
                                 short *set));
 extern void bdf_add_selection __((bdf_glyph_grid_t *grid, bdf_bitmap_t *sel));
 
+/*
+ * Glyph grid misc functions.
+ */
+extern int bdf_grid_color_at __((bdf_glyph_grid_t *grid, short x, short y));
+
 /*
  * Graphical transformation functions.
  */
@@ -565,6 +646,10 @@ extern int bdf_shear_glyphs __((bdf_font_t *font, short degrees,
                                 bdf_callback_t callback, void *data,
                                 int unencoded));
 
+extern int bdf_embolden_glyphs __((bdf_font_t *font, long start, long end,
+                                   bdf_callback_t callback, void *data,
+                                   int unencoded));
+
 extern int bdf_little_endian __((void));
 
 #undef __
