@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@
 #define _h_bdfP
 
 /*
- * $Id: bdfP.h,v 1.1 1998/07/24 16:12:18 mleisher Exp $
+ * $Id: bdfP.h,v 1.5 1999/06/22 16:45:51 mleisher Exp $
  */
 
 #include "bdf.h"
@@ -54,11 +54,19 @@ extern double _bdf_cos_tbl[];
 extern double _bdf_sin_tbl[];
 extern double _bdf_tan_tbl[];
 
+/*
+ * Arrays of masks for test with different bits per pixel.
+ */
+extern unsigned char onebpp[];
+extern unsigned char twobpp[];
+extern unsigned char fourbpp[];
+
 /*
  * Simple routine for determining the ceiling.
  */
 extern short _bdf_ceiling __((double v));
 
+extern unsigned char *_bdf_strdup __((unsigned char *s, unsigned long len));
 extern void _bdf_memmove __((char *dest, char *src, unsigned long bytes));
 
 extern short _bdf_atos __((char *s, char **end, int base));
@@ -84,6 +92,11 @@ extern void _bdf_add_acmsg __((bdf_font_t *font, char *msg,
 extern void _bdf_add_comment __((bdf_font_t *font, char *comment,
                                  unsigned long len));
 
+/*
+ * Function to do glyph name table cleanup when exiting.
+ */
+extern void _bdf_glyph_name_cleanup __((void));
+
 #undef __
 
 #ifdef __cplusplus
