@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: bdfcons.c,v 1.1 1998/07/24 16:12:18 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: bdfcons.c,v 1.4 1999/05/03 17:17:52 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: bdfcons.c,v 1.1 1998/07/24 16:12:18 mleisher Exp $";
+static char rcsid[] = "$Id: bdfcons.c,v 1.4 1999/05/03 17:17:52 mleisher Exp $";
 #endif
 #endif
 
@@ -150,7 +150,12 @@ int *awidth;
      * The point size of the font will be the height, the resolution will
      * default to 72dpi, and the spacing will default to proportional.
      */
-    fp = bdf_new_font(0, (long) hdr->max_height, 72, 72, BDF_PROPORTIONAL);
+    fp = bdf_new_font(0, (long) hdr->max_height, 72, 72, BDF_PROPORTIONAL, 1);
+
+    /*
+     * Force the bits per pixel to 1.
+     */
+    fp->bpp = 1;
 
     /*
      * Load the glyph metrics and set a marker to the beginning of the glyph
@@ -236,7 +241,7 @@ int *awidth;
      * routine returns.
      */
     if (ismono)
-      fp->spacing = BDF_MONOSPACE;
+      fp->spacing = BDF_MONOWIDTH;
 
     /*
      * Set up to load the glyphs.
@@ -336,7 +341,12 @@ int *awidth;
      * The point size of the font will be the height, the resolution will
      * default to 72dpi, and the spacing will default to character cell.
      */
-    fp = bdf_new_font(0, (long) hdr->height, 72, 72, BDF_CHARCELL);
+    fp = bdf_new_font(0, (long) hdr->height, 72, 72, BDF_CHARCELL, 1);
+
+    /*
+     * Force the bits per pixel to be 1.
+     */
+    fp->bpp = 1;
 
     /*
      * Make sure the width is always set to 8 no matter what.  This may
@@ -522,7 +532,12 @@ int type, *awidth;
      * The point size of the font will be the height, the resolution will
      * default to 72dpi, and the spacing will default to character cell.
      */
-    fp = bdf_new_font(0, (long) height, 72, 72, BDF_CHARCELL);
+    fp = bdf_new_font(0, (long) height, 72, 72, BDF_CHARCELL, 1);
+
+    /*
+     * Force the bits per pixel to be one.
+     */
+    fp->bpp = 1;
 
     /*
      * Make sure the width is always set to 8 no matter what.  This may
@@ -849,7 +864,7 @@ int *nfonts;
         prop.value.atom = "P";
         switch (fonts[res]->spacing) {
           case BDF_PROPORTIONAL: prop.value.atom = "P"; break;
-          case BDF_MONOSPACE: prop.value.atom = "M"; break;
+          case BDF_MONOWIDTH: prop.value.atom = "M"; break;
           case BDF_CHARCELL: prop.value.atom = "C"; break;
         }
         bdf_add_font_property(fonts[res], &prop);
