@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: bdffnt.c,v 1.1 1998/07/24 16:12:19 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: bdffnt.c,v 1.6 1999/10/23 13:43:36 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: bdffnt.c,v 1.1 1998/07/24 16:12:19 mleisher Exp $";
+static char rcsid[] = "$Id: bdffnt.c,v 1.6 1999/10/23 13:43:36 mleisher Exp $";
 #endif
 #endif
 
@@ -301,7 +301,14 @@ _bdffnt_get_short(field)
 char *field;
 #endif
 {
-    return (*field++ & 0xff) | ((*field++ & 0xff) << 8);
+    int a = 0, b = 1;
+
+    if (bdf_little_endian()) {
+        a = 1;
+        b = 0;
+    }
+
+    return (field[a] & 0xff) | ((field[b] & 0xff) << 8);
 }
 
 static unsigned long
@@ -312,8 +319,17 @@ _bdffnt_get_long(field)
 char *field;
 #endif
 {
-    return (*field++ & 0xff) | ((*field++ & 0xff) << 8) |
-        ((*field++ & 0xff) << 16) | ((*field++ & 0xff) << 24);
+    int a = 0, b = 1, c = 2, d = 3;
+
+    if (bdf_little_endian()) {
+        a = 3;
+        b = 2;
+        c = 1;
+        d = 0;
+    }
+
+    return (field[a] & 0xff) | ((field[b] & 0xff) << 8) |
+        ((field[c] & 0xff) << 16) | ((field[d] & 0xff) << 24);
 }
 
 /*
@@ -891,6 +907,7 @@ bdf_font_t **out;
     /*
      * Set some defaults.
      */
+    f->bpp = 1;
     f->default_glyph = font->info.dfDefaultChar + font->info.dfFirstChar;
     f->spacing = (font->info.dfFlags & 1) ? BDF_CHARCELL : BDF_PROPORTIONAL;
     f->glyphs_size = nchars;
@@ -976,19 +993,6 @@ bdf_font_t **out;
             }
         }
 
-#if 0
-        for (y = 0; y < gp->bbx.height; y++) {
-            for (x = 0; x < gp->bbx.width; x++) {
-                if (gp->bitmap[(y * bpr) + (x >> 3)] & (0x80 >> (x & 7)))
-                  putchar('*');
-                else
-                  putchar('.');
-            }
-            putchar('\n');
-        }
-        getchar();
-#endif
-
         if (callback != 0) {
             cb.reason = BDF_LOADING;
             cb.current = i;
@@ -1101,7 +1105,7 @@ bdf_font_t **out;
     prop.value.atom = "P";
     switch (f->spacing) {
       case BDF_PROPORTIONAL: prop.value.atom = "P"; break;
-      case BDF_MONOSPACE: prop.value.atom = "M"; break;
+      case BDF_MONOWIDTH: prop.value.atom = "M"; break;
       case BDF_CHARCELL: prop.value.atom = "C"; break;
     }
     bdf_add_font_property(f, &prop);
