@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,9 +28,9 @@
 
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: bdfttf.c,v 1.1 1998/07/24 16:12:22 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: bdfttf.c,v 1.5 1999/05/03 17:17:54 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: bdfttf.c,v 1.1 1998/07/24 16:12:22 mleisher Exp $";
+static char rcsid[] = "$Id: bdfttf.c,v 1.5 1999/05/03 17:17:54 mleisher Exp $";
 #endif
 #endif
 
@@ -205,7 +205,12 @@ char *name;
             for (i = 1; i < slen; i += 2) {
                 if (dash_to_space)
                   *name++ = (s[i] != '-') ? s[i] : ' ';
-                else
+                else if (s[i] == '\r' || s[i] == '\n') {
+                    if (s[i] == '\r' && i + 2 < slen && s[i + 2] == '\n')
+                      i += 2;
+                    *name++ = ' ';
+                    *name++ = ' ';
+                } else
                   *name++ = s[i];
             }
             *name = 0;
@@ -236,7 +241,12 @@ char *name;
         for (i = 1; i < slen; i += 2) {
             if (dash_to_space)
               *name++ = (s[i] != '-') ? s[i] : ' ';
-            else
+            else if (s[i] == '\r' || s[i] == '\n') {
+                if (s[i] == '\r' && i + 2 < slen && s[i + 2] == '\n')
+                  i += 2;
+                *name++ = ' ';
+                *name++ = ' ';
+            } else
               *name++ = s[i];
         }
         *name = 0;
@@ -540,7 +550,7 @@ bdf_font_t *fp;
      * Determine if the font is monowidth.
      */
     if (ismono) {
-        fp->spacing = BDF_MONOSPACE;
+        fp->spacing = BDF_MONOWIDTH;
         fp->monowidth = fp->bbx.width;
     }
 
@@ -589,7 +599,7 @@ bdf_font_t *fp;
     prop.value.atom = "P";
     switch (fp->spacing) {
       case BDF_PROPORTIONAL: prop.value.atom = "P"; break;
-      case BDF_MONOSPACE: prop.value.atom = "M"; break;
+      case BDF_MONOWIDTH: prop.value.atom = "M"; break;
       case BDF_CHARCELL: prop.value.atom = "C"; break;
     }
     bdf_add_font_property(fp, &prop);
@@ -677,6 +687,7 @@ bdf_font_t **font;
      * allocate at least the reported number of glyphs so reallocations will
      * be minimal.
      */
+    fp->bpp = 1;
     fp->default_glyph = -1;
     fp->spacing = BDF_PROPORTIONAL;
     fp->glyphs_size = properties->num_Glyphs;
