@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: finfo.c,v 1.1 1998/07/24 16:12:23 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: finfo.c,v 1.4 1999/05/03 17:17:55 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: finfo.c,v 1.1 1998/07/24 16:12:23 mleisher Exp $";
+static char rcsid[] = "$Id: finfo.c,v 1.4 1999/05/03 17:17:55 mleisher Exp $";
 #endif
 #endif
 
@@ -110,7 +110,7 @@ XtPointer client_data, call_data;
     if (XmToggleButtonGetState(ed->finfo.prop) == True)
       info.spacing = BDF_PROPORTIONAL;
     else if (XmToggleButtonGetState(ed->finfo.mono) == True)
-      info.spacing = BDF_MONOSPACE;
+      info.spacing = BDF_MONOWIDTH;
     else
       info.spacing = BDF_CHARCELL;
 
@@ -346,6 +346,9 @@ MXFEditor *ed;
     XtAddCallback(ed->finfo.vres, XmNvalueChangedCallback,
                   EnableUpdate, (XtPointer) ed->id);
 
+    ed->finfo.bpp = XtCreateManagedWidget("Bits Per Pixel: 0",
+                                          xmLabelWidgetClass, rc, 0, 0);
+
     ac = 0;
     XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
     XtSetArg(av[ac], XmNtopWidget, frame); ac++;
@@ -371,7 +374,7 @@ MXFEditor *ed;
     XtAddCallback(ed->finfo.prop, XmNvalueChangedCallback, DoSpacingChange,
                   (XtPointer) ed->id);
 
-    ed->finfo.mono = XtCreateManagedWidget("Monospace",
+    ed->finfo.mono = XtCreateManagedWidget("Monowidth",
                                            xmToggleButtonWidgetClass, rc,
                                            0, 0);
     XtAddCallback(ed->finfo.mono, XmNvalueChangedCallback, DoSpacingChange,
@@ -497,6 +500,15 @@ XtPointer client_data, call_data;
     sprintf(nbuf, "%ld", info.resolution_y);
     XmTextFieldSetString(ed->finfo.vres, nbuf);
 
+    /*
+     * Set the bits per pixel.
+     */
+    sprintf(name, "Bits Per Pixel: %ld", info.bits_per_pixel);
+    s = XmStringCreateSimple(name);
+    XtSetArg(av[0], XmNlabelString, s);
+    XtSetValues(ed->finfo.bpp, av, 1);
+    XmStringFree(s);
+
     /*
      * Set the font spacing type.
      */
@@ -505,7 +517,7 @@ XtPointer client_data, call_data;
         XmToggleButtonSetState(ed->finfo.prop, True, False);
         XtSetSensitive(ed->finfo.dwidth, False);
         break;
-      case BDF_MONOSPACE:
+      case BDF_MONOWIDTH:
         XmToggleButtonSetState(ed->finfo.mono, True, False);
         XtSetSensitive(ed->finfo.dwidth, True);
         break;
