@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: fntin.c,v 1.1 1998/07/24 16:12:24 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: fntin.c,v 1.3 1999/10/16 20:55:36 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: fntin.c,v 1.1 1998/07/24 16:12:24 mleisher Exp $";
+static char rcsid[] = "$Id: fntin.c,v 1.3 1999/10/16 20:55:36 mleisher Exp $";
 #endif
 #endif
 
@@ -329,9 +329,18 @@ bdf_font_t **out;
              * Now load each font in turn.
              */
             for (nfonts = i = 0; i < pcnt; i++) {
-                if (bdffnt_load_font(fnt, plist[i] - 1, LocalUpdateProgressBar,
-                                     (void *) &up, &flist[nfonts]) == 0)
-                  nfonts++;
+                if (xmbdfed_opts.progbar) {
+                    if (bdffnt_load_font(fnt, plist[i] - 1,
+                                         LocalUpdateProgressBar,
+                                         (void *) &up, &flist[nfonts]) == 0)
+                      nfonts++;
+                } else {
+                    WatchCursor(ed->fgrid, True);
+                    if (bdffnt_load_font(fnt, plist[i] - 1, 0, 0,
+                                         &flist[nfonts]) == 0)
+                      nfonts++;
+                    WatchCursor(ed->fgrid, False);
+                }
             }
 
             /*
@@ -506,12 +515,22 @@ bdf_font_t **out;
     /*
      * Only one font needs to be loaded, so handle that here.
      */
-    if (bdffnt_load_font(fnt, 0, UpdateProgressBar, (void *) file, &f) != 0) {
-        sprintf(name, "Problem loading font '%s'.\n", file);
-        ErrorDialog(name);
-        return 0;
+    if (xmbdfed_opts.progbar) {
+        if (bdffnt_load_font(fnt, 0, UpdateProgressBar, (void *) file,
+                             &f) != 0) {
+            sprintf(name, "Problem loading font '%s'.\n", file);
+            ErrorDialog(name);
+            return 0;
+        }
+    } else {
+        WatchCursor(ed->fgrid, True);
+        if (bdffnt_load_font(fnt, 0, 0, 0, &f) != 0) {
+            sprintf(name, "Problem loading font '%s'.\n", file);
+            ErrorDialog(name);
+            return 0;
+        }
+        WatchCursor(ed->fgrid, False);
     }
-
     /*
      * Munge the path name so the new extension can be added.
      */
