@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: glyphed.c,v 1.3 1998/10/12 18:46:43 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: glyphed.c,v 1.9 1999/10/16 20:55:36 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: glyphed.c,v 1.3 1998/10/12 18:46:43 mleisher Exp $";
+static char rcsid[] = "$Id: glyphed.c,v 1.9 1999/10/16 20:55:36 mleisher Exp $";
 #endif
 #endif
 
@@ -184,6 +184,7 @@ XtPointer client_data, call_data;
     bdf_glyph_t *glyph;
     char *s;
     bdf_metrics_t metrics;
+    int toggle;
     Boolean unencoded;
 
     ge = &glyph_editors[(unsigned long) client_data];
@@ -193,8 +194,14 @@ XtPointer client_data, call_data;
      * If the glyph editor has a selection, change back to the drawing
      * operation so the selection will be bound before saving.
      */
-    if (XmuttGlyphEditHasSelection(ge->gedit) == True)
-      XmuttGlyphEditToolboxSetToggle(ge->tbox, XmuttGLYPHEDIT_DRAW, True);
+    if (XmuttGlyphEditHasSelection(ge->gedit) == True) {
+        toggle = XmuttGlyphEditToolboxQueryToggle(ge->tbox);
+        if (toggle != XmuttGLYPHEDIT_DRAW) {
+            XmuttGlyphEditToolboxSetToggle(ge->tbox, XmuttGLYPHEDIT_DRAW,
+                                           True);
+            XmuttGlyphEditToolboxSetToggle(ge->tbox, toggle, True);
+        }
+    }
 
     glyph = XmuttGlyphEditGlyph(ge->gedit, &unencoded);
 
@@ -420,6 +427,22 @@ XtPointer client_data, call_data;
     XmuttGlyphEditShear(ge->gedit, cb->angle);
 }
 
+static void
+#ifndef _NO_PROTO
+EmboldenGlyph(Widget w, XtPointer client_data, XtPointer call_data)
+#else
+EmboldenGlyph(w, client_data, call_data)
+Widget w;
+XtPointer client_data, call_data;
+#endif
+{
+    MXFEditorGlyphEdit *ge;
+
+    ge = &glyph_editors[(unsigned long) client_data];
+
+    XmuttGlyphEditEmbolden(ge->gedit);
+}
+
 static void
 #ifndef _NO_PROTO
 ShowRotate(Widget w, XtPointer client_data, XtPointer call_data)
@@ -591,7 +614,7 @@ XtPointer client_data, call_data;
 
     if (ge->resize.shell == 0) {
         ac = 0;
-        sprintf(title, "%s - Glyph Edit Resize", app_name);
+        sprintf(title, "%s - Glyph Edit Resize BBX", app_name);
         XtSetArg(av[ac], XmNtitle, title); ac++;
         XtSetArg(av[ac], XmNallowShellResize, True); ac++;
         XtSetArg(av[ac], XmNdeleteResponse, XmUNMAP); ac++;
@@ -918,6 +941,7 @@ XtPointer client_data, call_data;
      * Set the glyph info on the glyph editor.
      */
     XmTextFieldSetString(ge->name, grid->name);
+    XmTextFieldSetCursorPosition(ge->name, strlen(grid->name));
 
     if (grid->unencoded)
       sprintf(name, "-1");
@@ -936,7 +960,7 @@ XtPointer client_data, call_data;
     sprintf(name, "%hd", grid->dwidth);
     XtSetSensitive(ge->dwidth, True);
     XmTextFieldSetString(ge->dwidth, name);
-    XmTextSetCursorPosition(ge->dwidth, 4);
+    XmTextFieldSetCursorPosition(ge->dwidth, 4);
     if (grid->spacing != BDF_PROPORTIONAL)
       XtSetSensitive(ge->dwidth, False);
 
@@ -1026,6 +1050,7 @@ XtPointer client_data, call_data;
      * Set the glyph info on the glyph editor.
      */
     XmTextFieldSetString(ge->name, grid->name);
+    XmTextFieldSetCursorPosition(ge->name, strlen(grid->name));
 
     if (grid->unencoded)
       sprintf(name, "-1");
@@ -1152,7 +1177,10 @@ XtPointer client_data, call_data;
     ge = &glyph_editors[id];
     cb = (XmuttGlyphEditUpdateCallbackStruct *) call_data;
 
-    sprintf(cbuf, "(%d,%d)", cb->x, cb->y);
+    if (cb->color >= 0)
+      sprintf(cbuf, "(%d,%d) color %d", cb->x, cb->y, cb->color);
+    else
+      sprintf(cbuf, "(%d,%d)", cb->x, cb->y);
     s = XmStringCreateSimple(cbuf);
     XtVaSetValues(ge->pointer, XmNlabelString, s, 0);
     XmStringFree(s);
@@ -1173,7 +1201,10 @@ XtPointer client_data, call_data;
     ge = &glyph_editors[(unsigned long) client_data];
     cb = (XmuttGlyphEditUpdateCallbackStruct *) call_data;
 
-    XmuttGlyphEditToolboxSetToggle(ge->tbox, cb->operation, False);
+    if (cb->reason == XmuttGLYPHEDIT_COLOR_CHANGE)
+      XmuttGlyphEditToolboxSetColor(ge->tbox, cb->color);
+    else
+      XmuttGlyphEditToolboxSetToggle(ge->tbox, cb->operation, False);
 }
 
 static void
@@ -1287,6 +1318,7 @@ XtPointer client_data, call_data;
      * Restore the glyph name.
      */
     XmTextFieldSetString(ge->name, grid->name);
+    XmTextFieldSetCursorPosition(ge->name, strlen(grid->name));
 
     /*
      * Restore the metrics info.
@@ -1468,6 +1500,11 @@ XtPointer client_data, call_data;
     ge = &glyph_editors[id];
     cb = (XmuttGlyphEditToolboxCallbackStruct *) call_data;
 
+    if (cb->reason == XmuttGLYPHEDIT_COLOR) {
+        XmuttGlyphEditChangeColor(ge->gedit, cb->color);
+        return;
+    }
+
     switch (cb->op_action) {
       case XmuttGLYPHEDIT_DRAW:
       case XmuttGLYPHEDIT_MOVE:
@@ -1525,54 +1562,6 @@ XtPointer client_data, call_data;
     }
 }
 
-#if 0
-static void
-#ifndef _NO_PROTO
-DoInitialPosition(Widget w, XtPointer client_data, XtPointer call_data)
-#else
-DoInitialPosition(w, client_data, call_data)
-Widget w;
-XtPointer client_data, call_data;
-#endif
-{
-    unsigned long id;
-    MXFEditorGlyphEdit *ge;
-    MXFEditor *ed;
-    Position x, y, rx, ry;
-    Dimension eht, ewd, fht, fwd;
-    Cardinal ac;
-    Arg av[6];
-
-    id = (unsigned long) client_data;
-    ge = &glyph_editors[id];
-    ed = &editors[ge->owner];
-
-    ac = 0;
-    XtSetArg(av[ac], XmNx, &x); ac++;
-    XtSetArg(av[ac], XmNy, &y); ac++;
-    XtSetArg(av[ac], XmNwidth, &ewd); ac++;
-    XtSetArg(av[ac], XmNheight, &eht); ac++;
-    XtGetValues(ed->fgrid, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNwidth, &fwd); ac++;
-    XtSetArg(av[ac], XmNheight, &fht); ac++;
-    XtGetValues(ge->shell, av, ac);
-
-    XtTranslateCoords(ed->shell, x, y, &rx, &ry);
-
-    rx += (ewd >> 1) - (fwd >> 1);
-    ry += (eht >> 1) - (fht >> 1);
-
-    XtMoveWidget(w, rx, ry);
-
-    /*
-     * Remove the callback because it is not needed any more.
-     */
-    XtRemoveCallback(w, XmNpopupCallback, DoInitialPosition, client_data);
-}
-#endif
-
 static void
 #ifndef _NO_PROTO
 MakeMenuBar(Widget mbar, MXFEditorGlyphEdit *ge)
@@ -1710,7 +1699,7 @@ MXFEditorGlyphEdit *ge;
     XtSetArg(av[ac], XmNmnemonic, 'R'); ac++;
     XtSetArg(av[ac], XmNaccelerator, "Ctrl<Key>r"); ac++;
     XtSetArg(av[ac], XmNacceleratorText, s); ac++;
-    pb = XtCreateManagedWidget("Resize", xmPushButtonWidgetClass,
+    pb = XtCreateManagedWidget("Resize BBX", xmPushButtonWidgetClass,
                                pdown, av, ac);
     XmStringFree(s);
     XtAddCallback(pb, XmNactivateCallback, DoResize, (XtPointer) ge->id);
@@ -1844,6 +1833,16 @@ MXFEditorGlyphEdit *ge;
                                av, ac);
     XmStringFree(s);
     XtAddCallback(pb, XmNactivateCallback, ShowShear, (XtPointer) ge->id);
+
+    s = XmStringCreateSimple("Ctrl+H");
+    ac = 0;
+    XtSetArg(av[ac], XmNmnemonic, 'E'); ac++;
+    XtSetArg(av[ac], XmNaccelerator, "Ctrl<Key>h"); ac++;
+    XtSetArg(av[ac], XmNacceleratorText, s); ac++;
+    pb = XtCreateManagedWidget("Embolden", xmPushButtonWidgetClass, pdown,
+                               av, ac);
+    XmStringFree(s);
+    XtAddCallback(pb, XmNactivateCallback, EmboldenGlyph, (XtPointer) ge->id);
 }
 
 static void
@@ -1859,10 +1858,10 @@ MXFEditor *ed;
 #endif
 {
     Widget frame, label, name_label, form, form1, form2, mb;
-    bdf_bitmap_t image;
     XmString s;
     Dimension ht, wd;
     Cardinal ac;
+    bdf_bitmap_t image;
     Arg av[10];
 
     if (ed->file == 0)
@@ -1907,6 +1906,7 @@ MXFEditor *ed;
     ac = 0;
     XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
     sprintf(name, "xmbdfed_glyphedit%ld_info_frame", ge->id);
     frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
 
@@ -1984,6 +1984,7 @@ MXFEditor *ed;
     XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
     XtSetArg(av[ac], XmNtopWidget, frame); ac++;
     XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
     sprintf(name, "xmbdfed_glyphedit%ld_metrics_frame", ge->id);
     frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
 
@@ -2116,6 +2117,9 @@ MXFEditor *ed;
     XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNglyphGrid, grid); ac++;
     XtSetArg(av[ac], XmNdefaultPixelSize, xmbdfed_opts.pixel_size); ac++;
+    XtSetArg(av[ac], XmNshowXHeight, xmbdfed_opts.show_x_height); ac++;
+    XtSetArg(av[ac], XmNshowCapHeight, xmbdfed_opts.show_cap_height); ac++;
+    XtSetArg(av[ac], XmNcolorList, xmbdfed_opts.pixels); ac++;
     sprintf(name, "xmbdfed_glyphedit%ld_editor", ge->id);
     ge->gedit = XtCreateManagedWidget(name, xmuttGlyphEditWidgetClass, form2,
                                       av, ac);
@@ -2163,6 +2167,7 @@ MXFEditor *ed;
     XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNglyphImage, &image); ac++;
+    XtSetArg(av[ac], XmNcolorList, xmbdfed_opts.pixels); ac++;
     sprintf(name, "xmbdfed_glyphedit%ld_toolbox", ge->id);
     ge->tbox = XtCreateManagedWidget(name, xmuttGlyphEditToolboxWidgetClass,
                                      form2, av, ac);
@@ -2216,9 +2221,11 @@ XtPointer client_data, call_data;
       grid = bdf_make_glyph_grid(font, cb->start, 0);
 
     ge = GetGlyphEditor(ed->id);
-    if (ge->name == 0)
-      BuildEditor(ge, grid, base, ed);
-    else {
+    if (ge->name == 0) {
+        BuildEditor(ge, grid, base, ed);
+        XmTextFieldSetCursorPosition(ge->name, strlen(grid->name));
+        XmTextFieldSetCursorPosition(ge->dwidth, 4);
+    } else {
         /*
          * Update the dialog title.
          */
@@ -2248,6 +2255,7 @@ XtPointer client_data, call_data;
          * Set the glyph info on the glyph editor.
          */
         XmTextFieldSetString(ge->name, grid->name);
+        XmTextFieldSetCursorPosition(ge->name, strlen(grid->name));
 
         if (grid->unencoded)
           sprintf(name, "-1");
@@ -2387,23 +2395,6 @@ unsigned short spacing, monowidth;
     }
 }
 
-void
-#ifndef _NO_PROTO
-GlyphEditChangePixelSize(unsigned long pixel_size)
-#else
-GlyphEditChangePixelSize(pixel_size)
-unsigned long pixel_size;
-#endif
-{
-    unsigned long i;
-    MXFEditorGlyphEdit *ge;
-    Arg av[1];
-
-    XtSetArg(av[0], XmNdefaultPixelSize, pixel_size);
-    for (i = 0, ge = glyph_editors; i < num_glyph_editors; i++, ge++)
-      XtSetValues(ge->gedit, av, 1);
-}
-
 void
 #ifndef _NO_PROTO
 GlyphEditCreatePixmaps(MXFEditor *ed)
@@ -2452,6 +2443,36 @@ MXFEditor *ed;
     }
 }
 
+void
+#ifndef _NO_PROTO
+GlyphEditChangeSetupValues(unsigned long pixel_size, int show_cap_height,
+                           int show_x_height)
+#else
+GlyphEditChangeSetupValues(pixel_size, show_cap_height, show_x_height)
+unsigned long pixel_size;
+int show_cap_height, show_x_height;
+#endif
+{
+    unsigned long i;
+    MXFEditorGlyphEdit *ge;
+    Arg av[3];
+
+    XtSetArg(av[0], XmNdefaultPixelSize, pixel_size);
+
+    if (show_cap_height)
+      XtSetArg(av[1], XmNshowCapHeight, True);
+    else
+      XtSetArg(av[1], XmNshowCapHeight, False);
+
+    if (show_x_height)
+      XtSetArg(av[2], XmNshowXHeight, True);
+    else
+      XtSetArg(av[2], XmNshowXHeight, False);
+
+    for (i = 0, ge = glyph_editors; i < num_glyph_editors; i++, ge++)
+      XtSetValues(ge->gedit, av, 3);
+}
+
 void
 #ifndef _NO_PROTO
 GlyphEditClose(MXFEditor *ed)
