@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: grab.c,v 1.1 1998/07/24 16:12:26 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: grab.c,v 1.4 1999/10/16 20:55:36 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: grab.c,v 1.1 1998/07/24 16:12:26 mleisher Exp $";
+static char rcsid[] = "$Id: grab.c,v 1.4 1999/10/16 20:55:36 mleisher Exp $";
 #endif
 #endif
 
@@ -132,9 +132,16 @@ XtPointer client_data, call_data;
     /*
      * Load the font.
      */
-    font = bdf_load_server_font(XtDisplay(w), f, fname,
-                                &xmbdfed_opts.font_opts, UpdateProgressBar,
-                                "server font");
+    if (xmbdfed_opts.progbar)
+      font = bdf_load_server_font(XtDisplay(w), f, fname,
+                                  &xmbdfed_opts.font_opts, UpdateProgressBar,
+                                  "server font");
+    else {
+        WatchCursor(ed->fgrid, True);
+        font = bdf_load_server_font(XtDisplay(w), f, fname,
+                                    &xmbdfed_opts.font_opts, 0, 0);
+        WatchCursor(ed->fgrid, False);
+    }
     XFreeFont(XtDisplay(w), f);
 
     if (font == 0) {
@@ -503,7 +510,7 @@ BuildFontGrabber()
     XtSetArg(av[ac], XmNrightPosition, 2); ac++;
     pb = XtCreateManagedWidget("Filter", xmPushButtonWidgetClass, form,
                                av, ac);
-    XtAddCallback(grab.filter, XmNactivateCallback, DoFilter, 0);
+    XtAddCallback(pb, XmNactivateCallback, DoFilter, 0);
 
     ac = 0;
     XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
