@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: ops.c,v 1.1 1998/07/24 16:12:29 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: ops.c,v 1.3 1999/09/14 17:40:47 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: ops.c,v 1.1 1998/07/24 16:12:29 mleisher Exp $";
+static char rcsid[] = "$Id: ops.c,v 1.3 1999/09/14 17:40:47 mleisher Exp $";
 #endif
 #endif
 
@@ -68,9 +68,21 @@ typedef struct {
     XtPointer data;
 } MXFEditorOtherOp;
 
+typedef struct {
+    int reason;
+    Widget shell;
+    Widget form;
+    Widget sel;
+    Widget all;
+    Widget ok;
+    XtCallbackProc cback;
+    XtPointer data;
+} MXFEditorEmboldenOp;
+
 typedef struct {
     MXFEditorTranslateOp trans;
     MXFEditorOtherOp other;
+    MXFEditorEmboldenOp embold;
 } MXFEditorOperations;
 
 static MXFEditorOperations ops;
@@ -103,6 +115,20 @@ XtPointer client_data, call_data;
     XtPopdown(ops.other.shell);
 }
 
+static void
+#ifndef _NO_PROTO
+CancelEmbolden(Widget w, XtPointer client_data, XtPointer call_data)
+#else
+CancelEmbolden(w, client_data, call_data)
+Widget w;
+XtPointer client_data, call_data;
+#endif
+{
+    ops.embold.cback = 0;
+    ops.embold.data = 0;
+    XtPopdown(ops.embold.shell);
+}
+
 static void
 #ifndef _NO_PROTO
 ActivateTranslate(Widget w, XtPointer client_data, XtPointer call_data)
@@ -152,6 +178,25 @@ XtPointer client_data, call_data;
     XtPopdown(ops.other.shell);
 }
 
+static void
+#ifndef _NO_PROTO
+ActivateEmbolden(Widget w, XtPointer client_data, XtPointer call_data)
+#else
+ActivateEmbolden(w, client_data, call_data)
+Widget w;
+XtPointer client_data, call_data;
+#endif
+{
+    XmuttOperationCallbackStruct cb;
+
+    if (ops.embold.cback != 0) {
+        cb.reason = ops.embold.reason;
+        cb.all = XmToggleButtonGetState(ops.embold.all);
+        (*ops.embold.cback)(w, ops.embold.data, (XtPointer) &cb);
+    }
+    XtPopdown(ops.embold.shell);
+}
+
 static void
 #ifndef _NO_PROTO
 BuildTransDialog(void)
@@ -423,6 +468,105 @@ BuildOtherDialog()
     XtAddCallback(pb, XmNactivateCallback, CancelOther, 0);
 }
 
+static void
+#ifndef _NO_PROTO
+BuildEmboldenDialog(void)
+#else
+BuildEmboldenDialog()
+#endif
+{
+    Widget label, frame, form, rc, pb;
+    Cardinal ac;
+    Arg av[8];
+
+    sprintf(title, "%s - Embolden Glyphs", app_name);
+    ac = 0;
+    XtSetArg(av[ac], XmNtitle, title); ac++;
+    XtSetArg(av[ac], XmNallowShellResize, True); ac++;
+    XtSetArg(av[ac], XmNdeleteResponse, XmUNMAP); ac++;
+    ops.embold.shell = XtCreatePopupShell("xmbdfed_emboldops_shell",
+                                          topLevelShellWidgetClass, top,
+                                          av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNverticalSpacing, 5); ac++;
+    XtSetArg(av[ac], XmNmarginWidth, 2); ac++;
+    XtSetArg(av[ac], XmNmarginHeight, 2); ac++;
+    ops.embold.form = XtCreateWidget("xmbdfed_emboldops_form",
+                                     xmFormWidgetClass, ops.embold.shell,
+                                     av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    frame = XtCreateManagedWidget("xmbdfed_emboldops_frame",
+                                  xmFrameWidgetClass, ops.embold.form,
+                                  av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
+    label = XtCreateManagedWidget("Embolden Glyphs",
+                                  xmLabelWidgetClass, frame, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNorientation, XmHORIZONTAL); ac++;
+    XtSetArg(av[ac], XmNradioBehavior, True); ac++;
+    rc = XtCreateManagedWidget("xmbdfed_emboldops_rowcol",
+                               xmRowColumnWidgetClass,
+                               frame, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNset, True); ac++;
+    ops.embold.sel = XtCreateManagedWidget("Selected Glyphs",
+                                           xmToggleButtonWidgetClass, rc,
+                                           av, ac);
+
+    ops.embold.all = XtCreateManagedWidget("All Glyphs",
+                                           xmToggleButtonWidgetClass, rc,
+                                           0, 0);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
+    frame = XtCreateManagedWidget("xmbdfed_emboldops_button_frame",
+                                  xmFrameWidgetClass, ops.embold.form, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNfractionBase, 2); ac++;
+    form = XtCreateManagedWidget("xmbdfed_emboldops_button_form",
+                                 xmFormWidgetClass, frame, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_POSITION); ac++;
+    XtSetArg(av[ac], XmNleftPosition, 0); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_POSITION); ac++;
+    XtSetArg(av[ac], XmNrightPosition, 1); ac++;
+    ops.embold.ok = XtCreateManagedWidget("Embolden", xmPushButtonWidgetClass,
+                                          form, av, ac);
+    XtAddCallback(ops.embold.ok, XmNactivateCallback, ActivateEmbolden, 0);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_POSITION); ac++;
+    XtSetArg(av[ac], XmNleftPosition, 1); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_POSITION); ac++;
+    XtSetArg(av[ac], XmNrightPosition, 2); ac++;
+    pb = XtCreateManagedWidget("Cancel", xmPushButtonWidgetClass, form,
+                               av, ac);
+    XtAddCallback(pb, XmNactivateCallback, CancelEmbolden, 0);
+}
+
 void
 #ifndef _NO_PROTO
 ShowTranslateDialog(XtCallbackProc cback, XtPointer data,
@@ -541,3 +685,31 @@ Boolean toggles_sensitive;
     XtManageChild(ops.other.form);
     XtPopup(ops.other.shell, XtGrabNone);
 }
+
+void
+#ifndef _NO_PROTO
+ShowEmboldenDialog(XtCallbackProc cback, XtPointer data,
+                   Boolean toggles_sensitive)
+#else
+ShowEmoldenDialog(cback, data, toggles_sensitive)
+XtCallbackProc cback;
+XtPointer data;
+Boolean toggles_sensitive;
+#endif
+{
+    if (ops.other.shell == 0)
+      BuildEmboldenDialog();
+
+    ops.embold.reason = XMBDFED_EMBOLDEN;
+
+    /*
+     * Set the sensitivity of the toggles.
+     */
+    XtSetSensitive(ops.embold.sel, toggles_sensitive);
+    XtSetSensitive(ops.embold.all, toggles_sensitive);
+
+    ops.embold.cback = cback;
+    ops.embold.data = data;
+    XtManageChild(ops.embold.form);
+    XtPopup(ops.embold.shell, XtGrabNone);
+}
