@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: prog.c,v 1.1 1998/07/24 16:12:29 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: prog.c,v 1.4 1999/05/03 17:17:57 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: prog.c,v 1.1 1998/07/24 16:12:29 mleisher Exp $";
+static char rcsid[] = "$Id: prog.c,v 1.4 1999/05/03 17:17:57 mleisher Exp $";
 #endif
 #endif
 
@@ -79,14 +79,15 @@ void *data;
     char *op, *op1;
     XmString label;
     XtInputMask mask;
-    Arg av[4];
+    Arg av[5];
 
     if (progbar_shell == 0)
       MakeProgressDialog();
 
     if (cb->reason == BDF_LOAD_START || cb->reason == BDF_SAVE_START ||
         cb->reason == BDF_TRANSLATE_START || cb->reason == BDF_ROTATE_START ||
-        cb->reason == BDF_SHEAR_START) {
+        cb->reason == BDF_SHEAR_START || cb->reason == BDF_GLYPH_NAME_START ||
+        cb->reason == BDF_EXPORT_START) {
         switch (cb->reason) {
           case BDF_LOAD_START:
             op = "Load";
@@ -108,6 +109,14 @@ void *data;
             op = "Shear";
             op1 = "Shearing";
             break;
+          case BDF_GLYPH_NAME_START:
+            op = "Rename";
+            op1 = "Renaming";
+            break;
+          case BDF_EXPORT_START:
+            op = "Export";
+            op1 = "Exporting";
+            break;
         }
         if (cb->reason == BDF_LOAD_START || cb->reason == BDF_SAVE_START)
           sprintf(title, "%s - %s font %s", app_name, op, (char *) data);
@@ -116,7 +125,8 @@ void *data;
         XtSetArg(av[0], XmNtitle, title);
         XtSetValues(progbar_shell, av, 1);
 
-        if (cb->reason == BDF_LOAD_START || cb->reason == BDF_SAVE_START)
+        if (cb->reason == BDF_LOAD_START || cb->reason == BDF_SAVE_START ||
+            cb->reason == BDF_EXPORT_START)
           sprintf(name, "%s %s ...", op1, (char *) data);
         else
           sprintf(name, "%s ...", op1);
@@ -126,7 +136,8 @@ void *data;
         XtSetArg(av[1], XmNmaxValue, cb->total);
         XtSetArg(av[2], XmNcurrentValue, cb->current);
         XtSetArg(av[3], XmNlabelString, label);
-        XtSetValues(progbar, av, 4);
+        XtSetArg(av[4], XmNpercentOnly, xmbdfed_opts.percent_only);
+        XtSetValues(progbar, av, 5);
         XmStringFree(label);
         XtManageChild(progbar_manager);
         XtPopup(progbar_shell, XtGrabNone);
@@ -138,7 +149,8 @@ void *data;
         XtSetArg(av[0], XmNminValue, 0);
         XtSetArg(av[1], XmNmaxValue, cb->total);
         XtSetArg(av[2], XmNcurrentValue, cb->current);
-        XtSetValues(progbar, av, 3);
+        XtSetArg(av[3], XmNpercentOnly, xmbdfed_opts.percent_only);
+        XtSetValues(progbar, av, 4);
     }
 
     if (cb->current == cb->total)
