@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -22,9 +22,9 @@
  */
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: props.c,v 1.1 1998/07/24 16:12:30 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: props.c,v 1.4 1999/10/23 13:43:37 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: props.c,v 1.1 1998/07/24 16:12:30 mleisher Exp $";
+static char rcsid[] = "$Id: props.c,v 1.4 1999/10/23 13:43:37 mleisher Exp $";
 #endif
 #endif
 
@@ -248,6 +248,11 @@ XtPointer client_data, call_data;
      */
     bdf_add_font_property(font, &prop);
 
+    /*
+     * Alert the FontGrid that a property has been changed.
+     */
+    XmuttFontGridCallModifiedCallback(ed->fgrid);
+
     /*
      * If the list must be updated, then do so by hand.
      */
@@ -453,140 +458,7 @@ MXFEditor *ed;
         XtCreateWidget(name, xmFormWidgetClass, ed->props.shell, av, ac);
 
     ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
-    label = XtCreateManagedWidget("Properties", xmLabelWidgetClass, form,
-                                  av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, label); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNvisibleItemCount, 5); ac++;
-    XtSetArg(av[ac], XmNscrollBarDisplayPolicy, XmSTATIC); ac++;
-    XtSetArg(av[ac], XmNlistSizePolicy, XmVARIABLE); ac++;
-    sprintf(name, "xmbdfed_properties%ld_list", ed->id);
-    ed->props.plist = XmCreateScrolledList(form, name, av, ac);
-    XtManageChild(ed->props.plist);
-    XtAddCallback(ed->props.plist, XmNbrowseSelectionCallback,
-                  SelectProperty, (XtPointer) ed->id);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, ed->props.plist); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    sprintf(name, "xmbdfed_properties%ld_frame1", ed->id);
-    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
-    label = XtCreateManagedWidget("Property Name", xmLabelWidgetClass, frame,
-                                  av, ac);
-
-    sprintf(name, "xmbdfed_properties%ld_name", ed->id);
-    ed->props.pname = XtCreateManagedWidget(name, xmTextFieldWidgetClass,
-                                            frame, 0, 0);
-    XtAddCallback(ed->props.pname, XmNvalueChangedCallback, EnableUpdate,
-                  (XtPointer) ed->id);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    sprintf(name, "xmbdfed_properties%ld_frame2", ed->id);
-    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
-    label = XtCreateManagedWidget("Property Value", xmLabelWidgetClass, frame,
-                                  av, ac);
-
-    sprintf(name, "xmbdfed_properties%ld_value", ed->id);
-    ed->props.pvalue = XtCreateManagedWidget(name, xmTextFieldWidgetClass,
-                                             frame, 0, 0);
-    XtAddCallback(ed->props.pvalue, XmNvalueChangedCallback, EnableUpdate,
-                  (XtPointer) ed->id);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    sprintf(name, "xmbdfed_properties%ld_frame3", ed->id);
-    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
-    label = XtCreateManagedWidget("Property Type", xmLabelWidgetClass, frame,
-                                  av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNorientation, XmHORIZONTAL); ac++;
-    XtSetArg(av[ac], XmNnumColumns, 1); ac++;
-    XtSetArg(av[ac], XmNpacking, XmPACK_COLUMN); ac++;
-    XtSetArg(av[ac], XmNradioBehavior, True); ac++;
-    sprintf(name, "xmbdfed_properties%ld_types", ed->id);
-    rc = XtCreateManagedWidget(name, xmRowColumnWidgetClass, frame, av, ac);
-
-    ed->props.atom = XtCreateManagedWidget("Atom", xmToggleButtonWidgetClass,
-                                           rc, 0, 0);
-    XtAddCallback(ed->props.atom, XmNvalueChangedCallback, EnableUpdate,
-                  (XtPointer) ed->id);
-
-    ed->props.integer = XtCreateManagedWidget("Integer",
-                                              xmToggleButtonWidgetClass, rc,
-                                              0, 0);
-    XtAddCallback(ed->props.integer, XmNvalueChangedCallback, EnableUpdate,
-                  (XtPointer) ed->id);
-
-    ed->props.cardinal = XtCreateManagedWidget("Cardinal",
-                                               xmToggleButtonWidgetClass, rc,
-                                               0, 0);
-    XtAddCallback(ed->props.cardinal, XmNvalueChangedCallback, EnableUpdate,
-                  (XtPointer) ed->id);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
-    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
-    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
-    sprintf(name, "xmbdfed_properties%ld_frame4", ed->id);
-    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
-    label = XtCreateManagedWidget("Show Properties", xmLabelWidgetClass, frame,
-                                  av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNorientation, XmHORIZONTAL); ac++;
-    XtSetArg(av[ac], XmNnumColumns, 1); ac++;
-    XtSetArg(av[ac], XmNpacking, XmPACK_COLUMN); ac++;
-    XtSetArg(av[ac], XmNradioBehavior, True); ac++;
-    sprintf(name, "xmbdfed_properties%ld_show_options", ed->id);
-    rc = XtCreateManagedWidget(name, xmRowColumnWidgetClass, frame, av, ac);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNset, True); ac++;
-    ed->props.fontprops = XtCreateManagedWidget("Font Properties",
-                                                xmToggleButtonWidgetClass,
-                                                rc, av, ac);
-    XtAddCallback(ed->props.fontprops, XmNvalueChangedCallback,
-                  ShowFontProperties, (XtPointer) ed->id);
-    ed->props.allprops = XtCreateManagedWidget("All Properties",
-                                               xmToggleButtonWidgetClass,
-                                               rc, 0, 0);
-    XtAddCallback(ed->props.allprops, XmNvalueChangedCallback,
-                  ShowAllProperties, (XtPointer) ed->id);
-
-    ac = 0;
-    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
-    XtSetArg(av[ac], XmNtopWidget, frame); ac++;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_NONE); ac++;
     XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
     XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_FORM); ac++;
@@ -633,6 +505,143 @@ MXFEditor *ed;
                                av, ac);
     XtAddCallback(pb, XmNactivateCallback, DoCloseProperties,
                   (XtPointer) ed->id);
+
+    form = ed->props.form;
+
+    ac = 0;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNbottomWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    sprintf(name, "xmbdfed_properties%ld_frame4", ed->id);
+    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
+    label = XtCreateManagedWidget("Show Properties", xmLabelWidgetClass, frame,
+                                  av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNorientation, XmHORIZONTAL); ac++;
+    XtSetArg(av[ac], XmNnumColumns, 1); ac++;
+    XtSetArg(av[ac], XmNpacking, XmPACK_COLUMN); ac++;
+    XtSetArg(av[ac], XmNradioBehavior, True); ac++;
+    sprintf(name, "xmbdfed_properties%ld_show_options", ed->id);
+    rc = XtCreateManagedWidget(name, xmRowColumnWidgetClass, frame, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNset, True); ac++;
+    ed->props.fontprops = XtCreateManagedWidget("Font Properties",
+                                                xmToggleButtonWidgetClass,
+                                                rc, av, ac);
+    XtAddCallback(ed->props.fontprops, XmNvalueChangedCallback,
+                  ShowFontProperties, (XtPointer) ed->id);
+    ed->props.allprops = XtCreateManagedWidget("All Properties",
+                                               xmToggleButtonWidgetClass,
+                                               rc, 0, 0);
+    XtAddCallback(ed->props.allprops, XmNvalueChangedCallback,
+                  ShowAllProperties, (XtPointer) ed->id);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNbottomWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    sprintf(name, "xmbdfed_properties%ld_frame3", ed->id);
+    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
+    label = XtCreateManagedWidget("Property Type", xmLabelWidgetClass, frame,
+                                  av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNorientation, XmHORIZONTAL); ac++;
+    XtSetArg(av[ac], XmNnumColumns, 1); ac++;
+    XtSetArg(av[ac], XmNpacking, XmPACK_COLUMN); ac++;
+    XtSetArg(av[ac], XmNradioBehavior, True); ac++;
+    sprintf(name, "xmbdfed_properties%ld_types", ed->id);
+    rc = XtCreateManagedWidget(name, xmRowColumnWidgetClass, frame, av, ac);
+
+    ed->props.atom = XtCreateManagedWidget("Atom", xmToggleButtonWidgetClass,
+                                           rc, 0, 0);
+    XtAddCallback(ed->props.atom, XmNvalueChangedCallback, EnableUpdate,
+                  (XtPointer) ed->id);
+
+    ed->props.integer = XtCreateManagedWidget("Integer",
+                                              xmToggleButtonWidgetClass, rc,
+                                              0, 0);
+    XtAddCallback(ed->props.integer, XmNvalueChangedCallback, EnableUpdate,
+                  (XtPointer) ed->id);
+
+    ed->props.cardinal = XtCreateManagedWidget("Cardinal",
+                                               xmToggleButtonWidgetClass, rc,
+                                               0, 0);
+    XtAddCallback(ed->props.cardinal, XmNvalueChangedCallback, EnableUpdate,
+                  (XtPointer) ed->id);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNbottomWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    sprintf(name, "xmbdfed_properties%ld_frame2", ed->id);
+    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
+    label = XtCreateManagedWidget("Property Value", xmLabelWidgetClass, frame,
+                                  av, ac);
+
+    sprintf(name, "xmbdfed_properties%ld_value", ed->id);
+    ed->props.pvalue = XtCreateManagedWidget(name, xmTextFieldWidgetClass,
+                                             frame, 0, 0);
+    XtAddCallback(ed->props.pvalue, XmNvalueChangedCallback, EnableUpdate,
+                  (XtPointer) ed->id);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNbottomWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    sprintf(name, "xmbdfed_properties%ld_frame1", ed->id);
+    frame = XtCreateManagedWidget(name, xmFrameWidgetClass, form, av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNchildType, XmFRAME_TITLE_CHILD); ac++;
+    label = XtCreateManagedWidget("Property Name", xmLabelWidgetClass, frame,
+                                  av, ac);
+
+    sprintf(name, "xmbdfed_properties%ld_name", ed->id);
+    ed->props.pname = XtCreateManagedWidget(name, xmTextFieldWidgetClass,
+                                            frame, 0, 0);
+    XtAddCallback(ed->props.pname, XmNvalueChangedCallback, EnableUpdate,
+                  (XtPointer) ed->id);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNalignment, XmALIGNMENT_BEGINNING); ac++;
+    label = XtCreateManagedWidget("Properties", xmLabelWidgetClass, form,
+                                  av, ac);
+
+    ac = 0;
+    XtSetArg(av[ac], XmNtopAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNtopWidget, label); ac++;
+    XtSetArg(av[ac], XmNbottomAttachment, XmATTACH_WIDGET); ac++;
+    XtSetArg(av[ac], XmNbottomWidget, frame); ac++;
+    XtSetArg(av[ac], XmNleftAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNrightAttachment, XmATTACH_FORM); ac++;
+    XtSetArg(av[ac], XmNvisibleItemCount, 5); ac++;
+    XtSetArg(av[ac], XmNscrollBarDisplayPolicy, XmSTATIC); ac++;
+    XtSetArg(av[ac], XmNlistSizePolicy, XmVARIABLE); ac++;
+    sprintf(name, "xmbdfed_properties%ld_list", ed->id);
+    ed->props.plist = XmCreateScrolledList(form, name, av, ac);
+    XtManageChild(ed->props.plist);
+    XtAddCallback(ed->props.plist, XmNbrowseSelectionCallback,
+                  SelectProperty, (XtPointer) ed->id);
+
 }
 
 void
