@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -28,9 +28,9 @@
 
 #ifndef lint
 #ifdef __GNUC__
-static char rcsid[] __attribute__ ((unused)) = "$Id: ttfin.c,v 1.1 1998/07/24 16:12:31 mleisher Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Id: ttfin.c,v 1.3 1999/10/16 20:55:37 mleisher Exp $";
 #else
-static char rcsid[] = "$Id: ttfin.c,v 1.1 1998/07/24 16:12:31 mleisher Exp $";
+static char rcsid[] = "$Id: ttfin.c,v 1.3 1999/10/16 20:55:37 mleisher Exp $";
 #endif
 #endif
 
@@ -388,10 +388,11 @@ BuildTTFDialog()
 
 int
 #ifdef __STDC__
-LoadTrueTypeFont(char **path, bdf_font_t **font)
+LoadTrueTypeFont(char **path, MXFEditor *ed, bdf_font_t **font)
 #else
-LoadTrueTypeFont(path, font)
+LoadTrueTypeFont(path, ed, font)
 char **path;
+MXFEditor *ed;
 bdf_font_t **font;
 #endif
 {
@@ -559,9 +560,16 @@ bdf_font_t **font;
             break;
         }
     }
-    res = bdfttf_load_font(face, &properties, pid, eid,
-                           &xmbdfed_opts.font_opts, UpdateProgressBar,
-                           (void *) file, font);
+    if (xmbdfed_opts.progbar)
+      res = bdfttf_load_font(face, &properties, pid, eid,
+                             &xmbdfed_opts.font_opts, UpdateProgressBar,
+                             (void *) file, font);
+    else {
+        WatchCursor(ed->fgrid, True);
+        res = bdfttf_load_font(face, &properties, pid, eid,
+                               &xmbdfed_opts.font_opts, 0, 0, font);
+        WatchCursor(ed->fgrid, False);
+    }
 
     /*
      * Close the typeface.
