@@ -1,5 +1,5 @@
 /*
- * Copyright 1996, 1997, 1998 Computing Research Labs,
+ * Copyright 1996, 1997, 1998, 1999 Computing Research Labs,
  * New Mexico State University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -24,7 +24,7 @@
 #define _h_xmbdfed
 
 /*
- * $Id: xmbdfed.h,v 1.3 1998/10/12 18:46:44 mleisher Exp $
+ * $Id: xmbdfed.h,v 1.21 1999/11/16 20:33:44 mleisher Exp $
  */
 
 #ifdef __cplusplus
@@ -41,7 +41,7 @@ extern "C" {
 #define __(x) ()
 #endif
 
-#define XMBDFED_VERSION "3.3"
+#define XMBDFED_VERSION "4.3"
 
 /**************************************************************************
  *
@@ -52,17 +52,24 @@ extern "C" {
 typedef struct {
     Widget save;
     Widget saveas;
+    Widget export;
     int open_type;
+    int export_type;
 } MXFEditorFileMenu;
 
 typedef struct {
     Widget cut;
     Widget copy;
     Widget paste;
+    Widget overlay;
+    Widget insert;
     Widget finfo;
     Widget dwidth;
     Widget xlfdname;
     Widget test;
+    Widget gname;
+    Widget unames;
+    Widget anames;
     Widget unfp;
     Widget upfn;
     Widget awidth;
@@ -73,6 +80,8 @@ typedef struct {
     Widget oct;
     Widget dec;
     Widget hex;
+    Widget vert;
+    Widget other;
     Widget acmsgs;
 } MXFEditorViewMenu;
 
@@ -87,6 +96,7 @@ typedef struct {
     Widget descent;
     Widget hres;
     Widget vres;
+    Widget bpp;
     Widget prop;
     Widget mono;
     Widget cell;
@@ -142,6 +152,8 @@ typedef struct {
     Widget fgrid;
     Widget elist;
     Boolean update_fontname;
+    long last_pgno;
+    long last_upgno;
     MXFEditorFileMenu filemenu;
     MXFEditorEditMenu editmenu;
     MXFEditorViewMenu viewmenu;
@@ -190,11 +202,25 @@ typedef struct {
 typedef struct {
     char *accelerator;
     char *accelerator_text;
+    char *unicode_name_file;
+    char *adobe_name_file;
     int no_blanks;
     int really_exit;
     int overwrite_mode;
+    int initial_glyph;
     unsigned long pixel_size;
     unsigned long resolution;
+    int show_cap_height;
+    int show_x_height;
+    int vertical;
+    int power2;
+    int progbar;
+    int percent_only;
+    int colors_allocated;
+    unsigned short colors[20];
+    unsigned long pixels[20];
+    int sbit;
+    int backups;
     bdf_options_t font_opts;
 } MXFEditorOptions;
 
@@ -215,6 +241,13 @@ typedef struct {
 #define XMBDFED_OPEN_TTF     4
 #define XMBDFED_OPEN_FNT     5
 
+/*
+ * Macros that indicate the behavior of the export file selection dialog when
+ * it is popped up.
+ */
+#define XMBDFED_EXPORT_PSF 1
+#define XMBDFED_EXPORT_HEX 2
+
 /**************************************************************************
  *
  * Global variables.
@@ -258,6 +291,12 @@ extern void ErrorDialog __((char *message));
 
 extern void DoClose __((Widget w, XtPointer client_data, XtPointer call_data));
 
+extern void WatchCursor __((Widget w, Boolean on));
+
+extern void xmbdfedAllocateColors __((Widget from, int bpp));
+extern void xmbdfedUpdateColorValues __((void));
+extern void xmbdfedRestoreColorValues __((void));
+
 /**************************************************************************
  *
  * Glyph editor functions.
@@ -274,7 +313,9 @@ extern void GlyphEditChangeSpacing __((unsigned long id,
                                        unsigned short spacing,
                                        unsigned short monowidth));
 
-extern void GlyphEditChangePixelSize __((unsigned long pixel_size));
+extern void GlyphEditChangeSetupValues __((unsigned long pixel_size,
+                                           int show_cap_height,
+                                           int show_x_height));
 
 extern void GlyphEditCreatePixmaps __((MXFEditor *ed));
 
@@ -316,6 +357,15 @@ extern void DoFontInfo __((Widget w, XtPointer client_data,
 extern void DoSetupDialog __((Widget w, XtPointer client_data,
                               XtPointer call_data));
 
+/**************************************************************************
+ *
+ * Color editor functions.
+ *
+ **************************************************************************/
+
+extern void DoColorDialog __((Widget w, XtPointer client_data,
+                              XtPointer call_data));
+
 /**************************************************************************
  *
  * Server font grabber functions.
@@ -370,6 +420,7 @@ extern void GlyphTestResize __((void));
 #define XMBDFED_TRANSLATE 1
 #define XMBDFED_ROTATE    2
 #define XMBDFED_SHEAR     3
+#define XMBDFED_EMBOLDEN  4
 
 /*
  * Special operation callback structure.
@@ -389,6 +440,8 @@ extern void ShowRotateDialog __((XtCallbackProc cback, XtPointer data,
                                  Boolean toggles_sensitive));
 extern void ShowShearDialog __((XtCallbackProc cback, XtPointer data,
                                 Boolean toggles_sensitive));
+extern void ShowEmboldenDialog __((XtCallbackProc cback, XtPointer data,
+                                   Boolean toggles_sensitive));
 
 #ifdef HAVE_FREETYPE
 
@@ -398,7 +451,8 @@ extern void ShowShearDialog __((XtCallbackProc cback, XtPointer data,
  *
  **************************************************************************/
 
-extern int LoadTrueTypeFont __((char **path, bdf_font_t **font));
+extern int LoadTrueTypeFont __((char **path, MXFEditor *ed,
+                                bdf_font_t **font));
 
 #endif /* HAVE_FREETYPE */
 
